@@ -21,17 +21,17 @@
   (PaCifiST will, however, read/write to these images as it does not perform
   FDC access as on a real ST)
 */
-char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.13 2003/10/11 20:14:33 thothy Exp $";
+char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.19 2004/07/01 20:54:10 thothy Exp $";
 
 #include <SDL_endian.h>
 
 #include "main.h"
 #include "debug.h"
 #include "configuration.h"
+#include "dim.h"
 #include "errlog.h"
 #include "file.h"
 #include "floppy.h"
-#include "memAlloc.h"
 #include "memorySnapShot.h"
 #include "misc.h"
 #include "msa.h"
@@ -40,13 +40,13 @@ char Floppy_rcsid[] = "Hatari $Id: floppy.c,v 1.13 2003/10/11 20:14:33 thothy Ex
 
 EMULATION_DRIVE EmulationDrives[NUM_EMULATION_DRIVES];  /* Emulation drive details, eg FileName, Inserted, Changed etc... */
 int nBootDrive=0;               /* Drive A, default */
-BOOL bFloppyChanged;            /* TRUE if a new floppy has been inserted */
 
 /* Possible disc image file extensions to scan for */
 char *pszDiscImageNameExts[] =
 {
   ".msa",
   ".st",
+  ".dim",
   NULL
 };
 
@@ -63,9 +63,7 @@ void Floppy_Init(void)
   for(i=0; i<NUM_EMULATION_DRIVES; i++)
   {
     /* Clear */
-    Memory_Clear(&EmulationDrives[i],sizeof(EMULATION_DRIVE));
-    /* And allocate buffer */
-    EmulationDrives[i].pBuffer = (unsigned char *)Memory_Alloc(DRIVE_BUFFER_BYTES);
+    memset(&EmulationDrives[i], 0, sizeof(EMULATION_DRIVE));
   }
 }
 
@@ -76,13 +74,7 @@ void Floppy_Init(void)
 */
 void Floppy_UnInit(void)
 {
-  int i;
-
-  /* Free buffers used for emulation drives */
-  for(i=0; i<NUM_EMULATION_DRIVES; i++)
-  {
-    Memory_Free(EmulationDrives[i].pBuffer);
-  }
+  Floppy_EjectBothDrives();
 }
 
 
@@ -94,13 +86,24 @@ void Floppy_MemorySnapShot_Capture(BOOL bSave)
 {
   int i;
 
+  /* If restoring then eject old drives first! */
+  if (!bSave)
+    Floppy_EjectBothDrives();
+
   /* Save/Restore details */
   for(i=0; i<NUM_EMULATION_DRIVES; i++)
   {
-    MemorySnapShot_Store(EmulationDrives[i].pBuffer,DRIVE_BUFFER_BYTES);
-    MemorySnapShot_Store(EmulationDrives[i].szFileName,sizeof(EmulationDrives[i].szFileName));
-    MemorySnapShot_Store(&EmulationDrives[i].nImageBytes,sizeof(EmulationDrives[i].nImageBytes));
     MemorySnapShot_Store(&EmulationDrives[i].bDiscInserted,sizeof(EmulationDrives[i].bDiscInserted));
+    MemorySnapShot_Store(&EmulationDrives[i].nImageBytes, sizeof(EmulationDrives[i].nImageBytes));
+    if (!bSave && EmulationDrives[i].bDiscInserted)
+    {
+      EmulationDrives[i].pBuffer = malloc(EmulationDrives[i].nImageBytes);
+      if (!EmulationDrives[i].pBuffer)
+        perror("Floppy_MemorySnapShot_Capture");
+    }
+    if (EmulationDrives[i].pBuffer)
+      MemorySnapShot_Store(EmulationDrives[i].pBuffer, EmulationDrives[i].nImageBytes);
+    MemorySnapShot_Store(EmulationDrives[i].szFileName, sizeof(EmulationDrives[i].szFileName));
     MemorySnapShot_Store(&EmulationDrives[i].bMediaChanged,sizeof(EmulationDrives[i].bMediaChanged));
     MemorySnapShot_Store(&EmulationDrives[i].bContentsChanged,sizeof(EmulationDrives[i].bContentsChanged));
     MemorySnapShot_Store(&EmulationDrives[i].bOKToSave,sizeof(EmulationDrives[i].bOKToSave));
@@ -132,7 +135,7 @@ void Floppy_GetBootDrive(void)
   To try and prevent data loss, we check for this error and flag the drive so
   the image will not be saved back to the file.
 */
-BOOL Floppy_IsBootSectorOK(int Drive)
+static BOOL Floppy_IsBootSectorOK(int Drive)
 {
   char szString[256];
   unsigned char *pDiscBuffer;
@@ -165,7 +168,7 @@ BOOL Floppy_IsBootSectorOK(int Drive)
   Try to create disc B filename, eg 'auto_100a' becomes 'auto_100b'
   Return TRUE if think we should try!
 */
-BOOL Floppy_CreateDiscBFileName(char *pSrcFileName, char *pDestFileName)
+static BOOL Floppy_CreateDiscBFileName(char *pSrcFileName, char *pDestFileName)
 {
   char szDir[256], szName[128], szExt[32];
 
@@ -203,8 +206,7 @@ BOOL Floppy_InsertDiscIntoDrive(int Drive, char *pszFileName)
 
 BOOL Floppy_ZipInsertDiscIntoDrive(int Drive, char *pszFileName, char *pszZipPath)
 {
-  char szDiscBFileName[MAX_FILENAME_LENGTH];
-  int nImageBytes=0;
+  long nImageBytes = 0;
 
   /* Eject disc, if one is inserted(don't inform user) */
   Floppy_EjectDiscFromDrive(Drive,FALSE);
@@ -213,18 +215,18 @@ BOOL Floppy_ZipInsertDiscIntoDrive(int Drive, char *pszFileName, char *pszZipPat
   if( !File_Exists(pszFileName) )
     File_FindPossibleExtFileName(pszFileName,pszDiscImageNameExts);
 
-  /* Is .MSA or .ST image? */
-  if (File_FileNameIsMSA(pszFileName))
-    nImageBytes = MSA_ReadDisc(pszFileName,EmulationDrives[Drive].pBuffer);
-  else if (File_FileNameIsST(pszFileName))
-    nImageBytes = ST_ReadDisc(pszFileName,EmulationDrives[Drive].pBuffer);
-  else if (File_FileNameIsZIP(pszFileName))
-    nImageBytes = ZIP_ReadDisc(pszFileName,pszZipPath,EmulationDrives[Drive].pBuffer);
-  else if (File_FileNameIsMSAGZ(pszFileName) || File_FileNameIsSTGZ(pszFileName))
-    nImageBytes = GZIP_ReadDisc(pszFileName,EmulationDrives[Drive].pBuffer);
+  /* Check disc image type and read the file: */
+  if (MSA_FileNameIsMSA(pszFileName, TRUE))
+    EmulationDrives[Drive].pBuffer = MSA_ReadDisc(pszFileName, &nImageBytes);
+  else if (ST_FileNameIsST(pszFileName, TRUE))
+    EmulationDrives[Drive].pBuffer = ST_ReadDisc(pszFileName, &nImageBytes);
+  else if (DIM_FileNameIsDIM(pszFileName, TRUE))
+    EmulationDrives[Drive].pBuffer = DIM_ReadDisc(pszFileName, &nImageBytes);
+  else if (ZIP_FileNameIsZIP(pszFileName))
+    EmulationDrives[Drive].pBuffer = ZIP_ReadDisc(pszFileName, pszZipPath, &nImageBytes);
 
   /* Did load OK? */
-  if (nImageBytes!=0)
+  if (EmulationDrives[Drive].pBuffer != NULL)
   {
     /* Store filename and size */
     strcpy(EmulationDrives[Drive].szFileName,pszFileName);
@@ -239,23 +241,20 @@ BOOL Floppy_ZipInsertDiscIntoDrive(int Drive, char *pszFileName, char *pszZipPat
   /* If we insert a disc into Drive A, should be try to put disc 2 into drive B? */
   if ( (Drive==0) && (ConfigureParams.DiscImage.bAutoInsertDiscB) )
   {
-    strcpy(EmulationDrives[1].szFileName,"");
+    char *szDiscBFileName = malloc(FILENAME_MAX);
     /* Attempt to make up second filename, eg was 'auto_100a' to 'auto_100b' */
-    if (Floppy_CreateDiscBFileName(pszFileName,szDiscBFileName))
+    if (szDiscBFileName && Floppy_CreateDiscBFileName(pszFileName, szDiscBFileName))
     {
       /* Put image into Drive B, clear out if fails */
       if (!Floppy_InsertDiscIntoDrive(1,szDiscBFileName))
         strcpy(EmulationDrives[1].szFileName,"");
     }
+    free(szDiscBFileName);
   }
 
-  bFloppyChanged = TRUE;
 
   /* Return TRUE if loaded OK */
-  if (nImageBytes)
-    return(TRUE);
-  else
-    return(FALSE);
+  return (EmulationDrives[Drive].pBuffer != NULL);
 }
 
 
@@ -277,16 +276,14 @@ void Floppy_EjectDiscFromDrive(int Drive, BOOL bInformUser)
       if (EmulationDrives[Drive].bOKToSave)
       {
         /* Save as .MSA or .ST image? */
-        if (File_FileNameIsMSA(EmulationDrives[Drive].szFileName))
-          MSA_WriteDisc(EmulationDrives[Drive].szFileName,EmulationDrives[Drive].pBuffer,EmulationDrives[Drive].nImageBytes);
-        else if (File_FileNameIsST(EmulationDrives[Drive].szFileName))
-          ST_WriteDisc(EmulationDrives[Drive].szFileName,EmulationDrives[Drive].pBuffer,EmulationDrives[Drive].nImageBytes);
-        else if (File_FileNameIsZIP(EmulationDrives[Drive].szFileName))
-          ZIP_WriteDisc(EmulationDrives[Drive].szFileName,EmulationDrives[Drive].pBuffer,EmulationDrives[Drive].nImageBytes);
-        else if (File_FileNameIsMSAGZ(EmulationDrives[Drive].szFileName) || 
-		 File_FileNameIsSTGZ(EmulationDrives[Drive].szFileName))
-          GZIP_WriteDisc(EmulationDrives[Drive].szFileName,EmulationDrives[Drive].pBuffer,EmulationDrives[Drive].nImageBytes);
-
+        if (MSA_FileNameIsMSA(EmulationDrives[Drive].szFileName, TRUE))
+          MSA_WriteDisc(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+        else if (ST_FileNameIsST(EmulationDrives[Drive].szFileName, TRUE))
+          ST_WriteDisc(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+        else if (DIM_FileNameIsDIM(EmulationDrives[Drive].szFileName, TRUE))
+          DIM_WriteDisc(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
+        else if (ZIP_FileNameIsZIP(EmulationDrives[Drive].szFileName))
+          ZIP_WriteDisc(EmulationDrives[Drive].szFileName, EmulationDrives[Drive].pBuffer, EmulationDrives[Drive].nImageBytes);
       }
     }
 
@@ -299,6 +296,11 @@ void Floppy_EjectDiscFromDrive(int Drive, BOOL bInformUser)
   }
 
   /* Drive is now empty */
+  if (EmulationDrives[Drive].pBuffer != NULL)
+  {
+    free(EmulationDrives[Drive].pBuffer);
+    EmulationDrives[Drive].pBuffer = NULL;
+  }
   strcpy(EmulationDrives[Drive].szFileName,"");
   EmulationDrives[Drive].nImageBytes = 0;
   EmulationDrives[Drive].bDiscInserted = FALSE;
@@ -369,20 +371,20 @@ static void Floppy_DoubleCheckFormat(long DiscSize, Uint16 *pnSides, Uint16 *pnS
 void Floppy_FindDiscDetails(unsigned char *pBuffer, int nImageBytes,
                             unsigned short int *pnSectorsPerTrack, unsigned short int *pnSides)
 {
-  unsigned char *pDiscBuffer;
-  Uint16 nSectorsPerTrack, nSides,nsectors;
+  Uint8 *pDiscBuffer;
+  Uint16 nSectorsPerTrack, nSides, nSectors;
 
   pDiscBuffer = pBuffer;
+
   /* First do check to find number of sectors and bytes per sector */
   nSectorsPerTrack = SDL_SwapLE16(*(Uint16 *)(pDiscBuffer+24));     /* SPT */
   nSides = SDL_SwapLE16(*(Uint16 *)(pDiscBuffer+26));               /* SIDE */
-  nsectors = SDL_SwapLE16(*(Uint16 *)(pDiscBuffer+19));             /* total sectors */
+  nSectors = pDiscBuffer[19] | (pDiscBuffer[20] << 8);              /* total sectors */
 
-  /* Now double-check info as boot-sector may contain incorrect information,
-     eg 'Eat.st' demo, or single/double sides */
-  if (nsectors != nImageBytes/512)
-    /* Don't be paranoid : if the number of sectors announced is correct, then chances
-       are high that the bootsector is correct */
+  /* If the number of sectors announced is incorrect, the boot-sector may
+   * contain incorrect information, eg the 'Eat.st' demo, or wrongly imaged
+   * single/double sided floppies... */
+  if (nSectors != nImageBytes/512)
     Floppy_DoubleCheckFormat(nImageBytes, &nSides, &nSectorsPerTrack);
 
   /* And set values */
@@ -403,22 +405,17 @@ BOOL Floppy_ReadSectors(int Drive,char *pBuffer,unsigned short int Sector,unsign
   unsigned char *pDiscBuffer;
   unsigned short int nSectorsPerTrack,nSides,nBytesPerTrack;
   long Offset;
-
-  if(Track > 85)
-  {
-    fprintf(stderr,"Floppy_ReadSectors: Strange floppy track (%i)!\n", Track);
-    return FALSE;
-  }
+  int nImageTracks;
 
   /* Do we have a disc in our drive? */
   if (EmulationDrives[Drive].bDiscInserted)
   {
     /* Looks good */
-    /*StatusBar_SetIcon(STATUS_ICON_FLOPPY,ICONSTATE_UPDATE);*/ /* Sorry - no statusbar in Hatari */
     pDiscBuffer = EmulationDrives[Drive].pBuffer;
 
     /* Find #sides and #sectors per track */
     Floppy_FindDiscDetails(EmulationDrives[Drive].pBuffer,EmulationDrives[Drive].nImageBytes,&nSectorsPerTrack,&nSides);
+    nImageTracks = ((EmulationDrives[Drive].nImageBytes / NUMBYTESPERSECTOR) / nSectorsPerTrack) / nSides;
 
     /* Need to read whole track? */
     if (Count<0)
@@ -443,6 +440,14 @@ BOOL Floppy_ReadSectors(int Drive,char *pBuffer,unsigned short int Sector,unsign
       return FALSE;
     }
 
+    /* Check if track number is in range */
+    if (Track >= nImageTracks)
+    {
+      fprintf(stderr, "Warning: Program tries to read from track %i of a disk "
+                      "image with only %i tracks!\n", Track, nImageTracks);
+      return FALSE;
+    }
+
     /* Seek to sector */
     nBytesPerTrack = NUMBYTESPERSECTOR*nSectorsPerTrack;
     Offset = nBytesPerTrack*Side;                 /* First seek to side */
@@ -474,16 +479,17 @@ BOOL Floppy_WriteSectors(int Drive,char *pBuffer,unsigned short int Sector,unsig
   unsigned char *pDiscBuffer;
   unsigned short int nSectorsPerTrack,nSides,nBytesPerTrack;
   long Offset;
+  int nImageTracks;
 
   /* Do we have a disc in our drive? */
   if (EmulationDrives[Drive].bDiscInserted)
   {
     /* Looks good */
-    /*StatusBar_SetIcon(STATUS_ICON_FLOPPY,ICONSTATE_UPDATE);*/ /* Sorry - no statusbar in Hatari yet */
     pDiscBuffer = EmulationDrives[Drive].pBuffer;
 
     /* Find #sides and #sectors per track */
     Floppy_FindDiscDetails(EmulationDrives[Drive].pBuffer,EmulationDrives[Drive].nImageBytes,&nSectorsPerTrack,&nSides);
+    nImageTracks = ((EmulationDrives[Drive].nImageBytes / NUMBYTESPERSECTOR) / nSectorsPerTrack) / nSides;
 
     /* Need to write whole track? */
     if (Count<0)
@@ -506,6 +512,14 @@ BOOL Floppy_WriteSectors(int Drive,char *pBuffer,unsigned short int Sector,unsig
       return FALSE;
     }
 
+    /* Check if track number is in range */
+    if (Track >= nImageTracks)
+    {
+      fprintf(stderr, "Warning: Program tries to write to track %i of a disk "
+                      "image with only %i tracks!\n", Track, nImageTracks);
+      return FALSE;
+    }
+
     /* Seek to sector */
     nBytesPerTrack = NUMBYTESPERSECTOR*nSectorsPerTrack;
     Offset = nBytesPerTrack*Side;               /* First seek to side */
