@@ -18,7 +18,7 @@
   * rmdir routine, can't remove dir with files in it. (another tos/unix difference)
   * Fix bugs, there are probably a few lurking around in here..
 */
-static char rcsid[] = "Hatari $Id: gemdos.c,v 1.24 2003/08/10 08:09:06 simonsunnyboy Exp $";
+char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.28 2004/12/08 10:27:53 thothy Exp $";
 
 #include <sys/stat.h>
 #include <time.h>
@@ -31,7 +31,6 @@ static char rcsid[] = "Hatari $Id: gemdos.c,v 1.24 2003/08/10 08:09:06 simonsunn
 #include "cart.h"
 #include "tos.h"
 #include "debug.h"
-#include "decode.h"
 #include "configuration.h"
 #include "file.h"
 #include "floppy.h"
@@ -44,8 +43,9 @@ static char rcsid[] = "Hatari $Id: gemdos.c,v 1.24 2003/08/10 08:09:06 simonsunn
 #include "printer.h"
 #include "rs232.h"
 #include "stMemory.h"
-
 #include "uae-cpu/hatari-glue.h"
+#include "uae-cpu/maccess.h"
+
 
 // #define GEMDOS_VERBOSE
 // uncomment the following line to debug filename lookups on hd
@@ -198,14 +198,14 @@ unsigned char GemDOS_ConvertAttribute(mode_t mode);
   Routines to convert time and date to MSDOS format.
   Originally from the STonX emulator. (cheers!)
 */
-unsigned short time2dos (time_t t)
+static Uint16 time2dos (time_t t)
 {
 	struct tm *x;
 	x = localtime (&t);
 	return (x->tm_sec>>1)|(x->tm_min<<5)|(x->tm_hour<<11);
 }
 
-unsigned short date2dos (time_t t)
+static Uint16 date2dos (time_t t)
 {
 	struct tm *x;
 	x = localtime (&t);
@@ -217,7 +217,7 @@ unsigned short date2dos (time_t t)
 /*
   Populate a DATETIME structure with file info
 */
-BOOL GetFileInformation(char *name, DATETIME *DateTime)
+static BOOL GetFileInformation(char *name, DATETIME *DateTime)
 {
   struct stat filestat;
   int n;
@@ -246,7 +246,7 @@ BOOL GetFileInformation(char *name, DATETIME *DateTime)
 /*
   Populate the DTA buffer with file info
 */
-int PopulateDTA(char *path, struct dirent *file)
+static BOOL PopulateDTA(char *path, struct dirent *file)
 {
   char tempstr[MAX_PATH];
   struct stat filestat;
@@ -259,9 +259,9 @@ int PopulateDTA(char *path, struct dirent *file)
   if(!pDTA) return(FALSE); /* no DTA pointer set */
   Misc_strupr(file->d_name);    /* convert to atari-style uppercase */
   strncpy(pDTA->dta_name,file->d_name,TOS_NAMELEN); /* FIXME: better handling of long file names */
-  STMemory_WriteLong_PCSpace(pDTA->dta_size, (long)filestat.st_size);
-  STMemory_WriteWord_PCSpace(pDTA->dta_time, time2dos(filestat.st_mtime));
-  STMemory_WriteWord_PCSpace(pDTA->dta_date, date2dos(filestat.st_mtime));
+  do_put_mem_long((uae_u32 *)(pDTA->dta_size), filestat.st_size);
+  do_put_mem_word((uae_u16 *)(pDTA->dta_time), time2dos(filestat.st_mtime));
+  do_put_mem_word((uae_u16 *)(pDTA->dta_date), date2dos(filestat.st_mtime));
   pDTA->dta_attrib = GemDOS_ConvertAttribute(filestat.st_mode);
 
   return(TRUE);
@@ -272,7 +272,8 @@ int PopulateDTA(char *path, struct dirent *file)
 /*
   Clear a used DTA structure.
 */
-void ClearInternalDTA(){
+static void ClearInternalDTA(void)
+{
   int i;
 
   /* clear the old DTA structure */
@@ -323,7 +324,8 @@ static int match (char *pat, char *name)
   Parse directory from sfirst mask
   - e.g.: input:  "hdemudir/auto/mask*.*" outputs: "hdemudir/auto"
 */
-void fsfirst_dirname(char *string, char *new){
+static void fsfirst_dirname(char *string, char *new)
+{
   int i=0;
 
   sprintf(new, string);
@@ -343,7 +345,8 @@ void fsfirst_dirname(char *string, char *new){
 /*
   Parse directory mask, e.g. "*.*"
 */
-void fsfirst_dirmask(char *string, char *new){
+static void fsfirst_dirmask(char *string, char *new)
+{
   int i=0, j=0;
   while(string[i] != '\0')i++;   /* go to end of string */
   while(string[i] != '/') i--;   /* find last slash */
@@ -352,6 +355,7 @@ void fsfirst_dirmask(char *string, char *new){
   new[j++] = '\0';
 }
 
+
 /*-----------------------------------------------------------------------*/
 /*
   Initialize GemDOS/PC file system
@@ -377,7 +381,7 @@ void GemDOS_Init(void)
 /*
   Reset GemDOS file system
 */
-void GemDOS_Reset()
+void GemDOS_Reset(void)
 {
   int i;
 
@@ -412,7 +416,7 @@ void GemDOS_Reset()
   Initialize a GEMDOS drive.
   Only 1 emulated drive allowed, as of yet.
 */
-void GemDOS_InitDrives()
+void GemDOS_InitDrives(void)
 {
   int i;
 
@@ -453,7 +457,7 @@ void GemDOS_InitDrives()
 /*
   Un-init the GEMDOS drive
 */
-void GemDOS_UnInitDrives()
+void GemDOS_UnInitDrives(void)
 {
   int i;
 
@@ -511,7 +515,7 @@ void GemDOS_MemorySnapShot_Capture(BOOL bSave)
 /*
   Return free PC file handle table index, or -1 if error
 */
-int GemDOS_FindFreeFileHandle(void)
+static int GemDOS_FindFreeFileHandle(void)
 {
   int i;
 
@@ -529,7 +533,7 @@ int GemDOS_FindFreeFileHandle(void)
 /*
   Check ST handle is within our table range, return TRUE if not
 */
-BOOL GemDOS_IsInvalidFileHandle(int Handle)
+static BOOL GemDOS_IsInvalidFileHandle(int Handle)
 {
   BOOL bInvalidHandle=FALSE;
 
@@ -547,7 +551,7 @@ BOOL GemDOS_IsInvalidFileHandle(int Handle)
   Find drive letter from a filename, eg C,D... and return as drive ID(C:2, D:3...)
   returns the current drive number if none is specified.
 */
-int GemDOS_FindDriveNumber(char *pszFileName)
+static int GemDOS_FindDriveNumber(char *pszFileName)
 {
   /* Does have 'A:' or 'C:' etc.. at start of string? */
   if ( (pszFileName[0]!='\0') && (pszFileName[1]==':') ) {
@@ -564,7 +568,7 @@ int GemDOS_FindDriveNumber(char *pszFileName)
 /*
   Return drive ID(C:2, D:3 etc...) or -1 if not one of our emulation hard-drives
 */
-int GemDOS_IsFileNameAHardDrive(char *pszFileName)
+static int GemDOS_IsFileNameAHardDrive(char *pszFileName)
 {
   int DriveLetter;
 
@@ -593,9 +597,14 @@ int GemDOS_IsFileNameAHardDrive(char *pszFileName)
   return(-1);
 }
 
-int baselen(char *s) {
-  /* Returns the length of the basename of the file passed in parameter
-     (ie the file without extension) */
+
+/*-----------------------------------------------------------------------*/
+/*
+  Returns the length of the basename of the file passed in parameter
+   (ie the file without extension)
+*/
+static int baselen(char *s)
+{
   char *ext = strchr(s,'.');
   if (ext) return ext-s;
   return strlen(s);
@@ -637,7 +646,8 @@ void GemDOS_CreateHardDriveFileName(int Drive,char *pszFileName,char *pszDestNam
     {
       glob_t globbuf;
       char old1,old2,dest[256];
-      int len,j,found,base_len;
+      int len, found, base_len;
+      unsigned int j;
 
       *start++ = '/';
       old1 = *start; *start++ = '*';
@@ -694,8 +704,9 @@ void GemDOS_CreateHardDriveFileName(int Drive,char *pszFileName,char *pszDestNam
     if (*start && !strchr(start,'?') && !strchr(start,'*')) {
       /* We have a complete name after the path, not a wildcard */
       glob_t globbuf;
-      char old1,old2,dest[256];
-      int len,j,found,base_len;
+      char old1,old2;
+      int len, found, base_len;
+      unsigned int j;
 
       old1 = *start; *start++ = '*';
       old2 = *start; *start = 0;
@@ -762,7 +773,8 @@ unsigned char GemDOS_ConvertAttribute(mode_t mode)
   GEMDOS Cauxin
   Call 0x3
 */
-BOOL GemDOS_Cauxin(unsigned long Params)
+#if 0
+static BOOL GemDOS_Cauxin(unsigned long Params)
 {
   unsigned char Char;
 
@@ -775,13 +787,15 @@ BOOL GemDOS_Cauxin(unsigned long Params)
 
   return(TRUE);
 }
+#endif
 
 /*-----------------------------------------------------------------------*/
 /*
   GEMDOS Cauxout
   Call 0x4
 */
-BOOL GemDOS_Cauxout(unsigned long Params)
+#if 0
+static BOOL GemDOS_Cauxout(unsigned long Params)
 {
   unsigned char Char;
 
@@ -791,13 +805,14 @@ BOOL GemDOS_Cauxout(unsigned long Params)
 
   return(TRUE);
 }
+#endif
 
 /*-----------------------------------------------------------------------*/
 /*
   GEMDOS Cprnout
   Call 0x5
 */
-BOOL GemDOS_Cprnout(unsigned long Params)
+static BOOL GemDOS_Cprnout(unsigned long Params)
 {
   unsigned char Char;
 
@@ -814,7 +829,7 @@ BOOL GemDOS_Cprnout(unsigned long Params)
   GEMDOS Set drive (0=A,1=B,2=C etc...)
   Call 0xE
 */
-BOOL GemDOS_SetDrv(unsigned long Params)
+static BOOL GemDOS_SetDrv(unsigned long Params)
 {
   /* Read details from stack for our own use */
   CurrentDrive = STMemory_ReadWord(Params+SIZE_WORD);
@@ -828,7 +843,7 @@ BOOL GemDOS_SetDrv(unsigned long Params)
   GEMDOS Cprnos
   Call 0x11
 */
-BOOL GemDOS_Cprnos(unsigned long Params)
+static BOOL GemDOS_Cprnos(unsigned long Params)
 {
   /* pritner status depends if printing is enabled or not... */
   if(ConfigureParams.Printer.bEnablePrinting)
@@ -844,7 +859,8 @@ BOOL GemDOS_Cprnos(unsigned long Params)
   GEMDOS Cauxis
   Call 0x12
 */
-BOOL GemDOS_Cauxis(unsigned long Params)
+#if 0
+static BOOL GemDOS_Cauxis(unsigned long Params)
 {
   /* Read our RS232 state */
   if (RS232_GetStatus())
@@ -854,25 +870,28 @@ BOOL GemDOS_Cauxis(unsigned long Params)
 
   return(TRUE);
 }
+#endif
 
 /*-----------------------------------------------------------------------*/
 /*
   GEMDOS Cauxos
   Call 0x13
 */
-BOOL GemDOS_Cauxos(unsigned long Params)
+#if 0
+static BOOL GemDOS_Cauxos(unsigned long Params)
 {
   Regs[REG_D0] = -1;                /* Device ready */
 
   return(TRUE);
 }
+#endif
 
 /*-----------------------------------------------------------------------*/
 /*
   GEMDOS Set Disc Transfer Address (DTA)
   Call 0x1A
 */
-BOOL GemDOS_SetDTA(unsigned long Params)
+static BOOL GemDOS_SetDTA(unsigned long Params)
 {
   /* Look up on stack to find where DTA is! Store as PC pointer */
   pDTA = (DTA *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
@@ -885,7 +904,7 @@ BOOL GemDOS_SetDTA(unsigned long Params)
   GEMDOS Dfree Free disc space.
   Call 0x39
 */
-BOOL GemDOS_DFree(unsigned long Params)
+static BOOL GemDOS_DFree(unsigned long Params)
 {
   int Drive;
   unsigned long Address;
@@ -910,7 +929,7 @@ BOOL GemDOS_DFree(unsigned long Params)
   GEMDOS MkDir
   Call 0x39
 */
-BOOL GemDOS_MkDir(unsigned long Params)
+static BOOL GemDOS_MkDir(unsigned long Params)
 {
   char szDirPath[MAX_PATH];
   char *pDirName;
@@ -941,7 +960,7 @@ BOOL GemDOS_MkDir(unsigned long Params)
   GEMDOS RmDir
   Call 0x3A
 */
-BOOL GemDOS_RmDir(unsigned long Params)
+static BOOL GemDOS_RmDir(unsigned long Params)
 {
   char szDirPath[MAX_PATH];
   char *pDirName;
@@ -970,12 +989,11 @@ BOOL GemDOS_RmDir(unsigned long Params)
   GEMDOS ChDir
   Call 0x3B
 */
-BOOL GemDOS_ChDir(unsigned long Params)
+static BOOL GemDOS_ChDir(unsigned long Params)
 {
   char szDirPath[MAX_PATH];
   char *pDirName;
   int Drive;
-  int n;
 
   /* Find new directory */
   pDirName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
@@ -987,7 +1005,6 @@ BOOL GemDOS_ChDir(unsigned long Params)
 
   if (ISHARDDRIVE(Drive)) {
 
-    char dest[256];
     struct stat buf;
 
     GemDOS_CreateHardDriveFileName(Drive,pDirName,szDirPath);
@@ -1015,7 +1032,7 @@ BOOL GemDOS_ChDir(unsigned long Params)
   GEMDOS Create file
   Call 0x3C
 */
-BOOL GemDOS_Create(unsigned long Params)
+static BOOL GemDOS_Create(unsigned long Params)
 {
   char szActualFileName[MAX_PATH];
   char *pszFileName;
@@ -1069,7 +1086,7 @@ BOOL GemDOS_Create(unsigned long Params)
   GEMDOS Open file
   Call 0x3D
 */
-BOOL GemDOS_Open(unsigned long Params)
+static BOOL GemDOS_Open(unsigned long Params)
 {
   char szActualFileName[MAX_PATH];
   char *pszFileName;
@@ -1115,7 +1132,7 @@ BOOL GemDOS_Open(unsigned long Params)
   GEMDOS Close file
   Call 0x3E
 */
-BOOL GemDOS_Close(unsigned long Params)
+static BOOL GemDOS_Close(unsigned long Params)
 {
   int Handle;
 
@@ -1142,7 +1159,7 @@ BOOL GemDOS_Close(unsigned long Params)
   GEMDOS Read file
   Call 0x3F
 */
-BOOL GemDOS_Read(unsigned long Params)
+static BOOL GemDOS_Read(unsigned long Params)
 {
   char *pBuffer;
   unsigned long nBytesRead,Size,CurrentPos,FileSize;
@@ -1195,7 +1212,7 @@ BOOL GemDOS_Read(unsigned long Params)
   GEMDOS Write file
   Call 0x40
 */
-BOOL GemDOS_Write(unsigned long Params)
+static BOOL GemDOS_Write(unsigned long Params)
 {
   char *pBuffer;
   unsigned long Size,nBytesWritten;
@@ -1234,7 +1251,7 @@ BOOL GemDOS_Write(unsigned long Params)
   GEMDOS UnLink(Delete) file
   Call 0x41
 */
-BOOL GemDOS_UnLink(unsigned long Params)
+static BOOL GemDOS_UnLink(unsigned long Params)
 {
 #ifdef ENABLE_SAVING
   char szActualFileName[MAX_PATH];
@@ -1266,7 +1283,7 @@ BOOL GemDOS_UnLink(unsigned long Params)
   GEMDOS File seek
   Call 0x42
 */
-BOOL GemDOS_LSeek(unsigned long Params)
+static BOOL GemDOS_LSeek(unsigned long Params)
 {
   long Offset;
   int Handle,Mode;
@@ -1294,7 +1311,7 @@ BOOL GemDOS_LSeek(unsigned long Params)
   GEMDOS Get Directory
   Call 0x47
 */
-int GemDOS_GetDir(unsigned long Params){
+static int GemDOS_GetDir(unsigned long Params){
   unsigned long Address;
   unsigned short Drive;
 
@@ -1327,7 +1344,7 @@ int GemDOS_GetDir(unsigned long Params){
 
   If loading from hard-drive(ie drive ID 2 or more) set condition codes to run own GEMDos routines
 */
-int GemDOS_Pexec_LoadAndGo(unsigned long Params)
+static int GemDOS_Pexec_LoadAndGo(unsigned long Params)
 {
   /* add multiple disk support here too */
   /* Hard-drive? */
@@ -1343,7 +1360,7 @@ int GemDOS_Pexec_LoadAndGo(unsigned long Params)
 /*
   PExec Load But Don't Go - Redirect to cart' routine at address 0xFA1000
 */
-int GemDOS_Pexec_LoadDontGo(unsigned long Params)
+static int GemDOS_Pexec_LoadDontGo(unsigned long Params)
 {
   /* Hard-drive? */
   if( CurrentDrive == emudrives[0]->hd_letter )
@@ -1357,7 +1374,7 @@ int GemDOS_Pexec_LoadDontGo(unsigned long Params)
   GEMDOS PExec handler
   Call 0x4B
 */
-int GemDOS_Pexec(unsigned long Params)
+static int GemDOS_Pexec(unsigned long Params)
 {
   unsigned short int Mode;
 
@@ -1391,7 +1408,7 @@ int GemDOS_Pexec(unsigned long Params)
   GEMDOS Search Next
   Call 0x4F
 */
-BOOL GemDOS_SNext(unsigned long Params)
+static BOOL GemDOS_SNext(unsigned long Params)
 {
   struct dirent **temp;
   int Index;
@@ -1400,10 +1417,10 @@ BOOL GemDOS_SNext(unsigned long Params)
   pDTA = (DTA *)STRAM_ADDR(STMemory_ReadLong(STMemory_ReadLong(act_pd)+32));
 
   /* Was DTA ours or TOS? */
-  if (STMemory_ReadLong_PCSpace(pDTA->magic)==DTA_MAGIC_NUMBER) {
+  if (do_get_mem_long((uae_u32 *)(pDTA->magic)) == DTA_MAGIC_NUMBER) {
 
     /* Find index into our list of structures */
-    Index = STMemory_ReadWord_PCSpace(pDTA->index)&(MAX_DTAS_FILES-1);
+    Index = do_get_mem_word((uae_u16 *)(pDTA->index))&(MAX_DTAS_FILES-1);
 
     if(InternalDTAs[Index].centry >= InternalDTAs[Index].nentries){
       Regs[REG_D0] = GEMDOS_ENMFIL;    /* No more files */
@@ -1429,7 +1446,7 @@ BOOL GemDOS_SNext(unsigned long Params)
   GEMDOS Find first file
   Call 0x4E
 */
-BOOL GemDOS_SFirst(unsigned long Params)
+static BOOL GemDOS_SFirst(unsigned long Params)
 {
   char szActualFileName[MAX_PATH];
   char tempstr[MAX_PATH];
@@ -1454,8 +1471,9 @@ BOOL GemDOS_SFirst(unsigned long Params)
     GemDOS_CreateHardDriveFileName(Drive,pszFileName,szActualFileName);
 
     /* Populate DTA, set index for our use */
-    STMemory_WriteWord_PCSpace(pDTA->index,DTAIndex);
-    STMemory_WriteLong_PCSpace(pDTA->magic,DTA_MAGIC_NUMBER); /* set our dta magic num */
+    do_put_mem_word((uae_u16 *)(pDTA->index), DTAIndex);
+    /* set our dta magic num */
+    do_put_mem_long((uae_u32 *)(pDTA->magic), DTA_MAGIC_NUMBER);
 
     if(InternalDTAs[DTAIndex].bUsed == TRUE) ClearInternalDTA();
     InternalDTAs[DTAIndex].bUsed = TRUE;
@@ -1533,7 +1551,7 @@ BOOL GemDOS_SFirst(unsigned long Params)
   GEMDOS Rename
   Call 0x56
 */
-BOOL GemDOS_Rename(unsigned long Params)
+static BOOL GemDOS_Rename(unsigned long Params)
 {
   char *pszNewFileName,*pszOldFileName;
   char szNewActualFileName[MAX_PATH],szOldActualFileName[MAX_PATH];
@@ -1566,7 +1584,7 @@ BOOL GemDOS_Rename(unsigned long Params)
   GEMDOS GSDToF
   Call 0x57
 */
-BOOL GemDOS_GSDToF(unsigned long Params)
+static BOOL GemDOS_GSDToF(unsigned long Params)
 {
   DATETIME DateTime;
   unsigned long pBuffer;
@@ -1602,8 +1620,8 @@ BOOL GemDOS_GSDToF(unsigned long Params)
 /*-----------------------------------------------------------------------*/
 /*
   Run GEMDos call, and re-direct if need to. Used to handle hard-disc emulation etc...
-  This sets the condition codes(in SR), which are used in the 'cart.s' program to decide if we
-  need to run old GEM vector, or PExec or nothing.
+  This sets the condition codes (in SR), which are used in the 'cart_asm.s' program to
+  decide if we need to run old GEM vector, or PExec or nothing.
 
   This method keeps the stack and other states consistant with the original ST which is very important
   for the PExec call and maximum compatibility through-out
@@ -1772,7 +1790,7 @@ void GemDOS_OpCode(void)
   Sets up our gemdos handler (or, if we don't need one, just turn off keyclicks)
  */
 
-void GemDOS_Boot()
+void GemDOS_Boot(void)
 {
 
   bInitGemDOS = TRUE;
@@ -1780,12 +1798,8 @@ void GemDOS_Boot()
   fprintf(stderr, "Gemdos_Boot()\n");
 #endif
   /* install our gemdos handler, if -e or --harddrive option used */
-  if(GEMDOS_EMU_ON){
-
-    /* Patch pexec code - coded value is 4, but should be 6 for TOS >= 1.04 */
-    if(TosVersion >= 0x0104)
-      STMemory_WriteByte(CART_PEXEC_TOS, 0x06);
-
+  if (GEMDOS_EMU_ON)
+  {
     /* Get the address of the p_run variable that points to the actual basepage */
     if(TosVersion == 0x100)
     {
@@ -1802,7 +1816,7 @@ void GemDOS_Boot()
 
     /* Save old GEMDOS handler adress */
     STMemory_WriteLong(CART_OLDGEMDOS, STMemory_ReadLong(0x0084));
-    /* Setup new GEMDOS handler, see cartimg.c */
+    /* Setup new GEMDOS handler, see "cart_asm.s" */
     STMemory_WriteLong(0x0084, CART_GEMDOS);
   }
 }
