@@ -6,7 +6,7 @@
 
   A tiny graphical user interface for Hatari.
 */
-static char rcsid[] = "Hatari $Id: sdlgui.c,v 1.2 2003/08/11 19:37:36 thothy Exp $";
+char SDLGui_rcsid[] = "Hatari $Id: sdlgui.c,v 1.6 2004/06/18 18:07:16 thothy Exp $";
 
 #include <SDL.h>
 #include <ctype.h>
@@ -17,30 +17,83 @@ static char rcsid[] = "Hatari $Id: sdlgui.c,v 1.2 2003/08/11 19:37:36 thothy Exp
 #include "screen.h"
 #include "sdlgui.h"
 
+#include "font5x8.h"
+#include "font10x16.h"
 
-static SDL_Surface *stdfontgfx = NULL;  /* The 'standard' font graphics */
-static SDL_Surface *fontgfx = NULL;     /* The actual font graphics */
-static int fontwidth, fontheight;       /* Height and width of the actual font */
 
+static SDL_Surface *pSmallFontGfx = NULL;   /* The small font graphics */
+static SDL_Surface *pBigFontGfx = NULL;     /* The big font graphics */
+static SDL_Surface *fontgfx = NULL;         /* The actual font graphics */
+static int fontwidth, fontheight;           /* Width & height of the actual font */
+
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Load an 1 plane XBM into a 8 planes SDL_Surface.
+*/
+static SDL_Surface *SDLGui_LoadXBM(int w, int h, Uint8 *srcbits)
+{
+  SDL_Surface *bitmap;
+  Uint8 *dstbits;
+  int x, y, srcpitch;
+  int mask;
+
+  /* Allocate the bitmap */
+  bitmap = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 8, 0, 0, 0, 0);
+  if (bitmap == NULL)
+  {
+    fprintf(stderr, "Couldn't allocate bitmap: %s", SDL_GetError());
+    return(NULL);
+  }
+
+  srcpitch = ((w + 7) / 8);
+  dstbits = (Uint8 *)bitmap->pixels;
+  mask = 1;
+
+  /* Copy the pixels */
+  for (y = 0 ; y < h ; y++)
+  {
+    for (x = 0 ; x < w ; x++)
+    {
+      dstbits[x] = (srcbits[x / 8] & mask) ? 1 : 0;
+      mask <<= 1;
+      mask |= (mask >> 8);
+      mask &= 0xFF;
+    }
+    dstbits += bitmap->pitch;
+    srcbits += srcpitch;
+  }
+
+  return(bitmap);
+}
 
 
 /*-----------------------------------------------------------------------*/
 /*
   Initialize the GUI.
 */
-int SDLGui_Init()
+int SDLGui_Init(void)
 {
-  char fontname[256];
-  sprintf(fontname, "%s/font8.bmp", DATADIR);
+  SDL_Color blackWhiteColors[2] = {{255, 255, 255, 0}, {0, 0, 0, 0}};
 
-  /* Load the font graphics: */
-  stdfontgfx = SDL_LoadBMP(fontname);
-  if( stdfontgfx == NULL )
+  /* Initialize the font graphics: */
+  pSmallFontGfx = SDLGui_LoadXBM(font5x8_width, font5x8_height, font5x8_bits);
+  pBigFontGfx = SDLGui_LoadXBM(font10x16_width, font10x16_height, font10x16_bits);
+  if (pSmallFontGfx == NULL || pBigFontGfx == NULL)
   {
-    fprintf(stderr, "Error: Can't load image %s: %s\n", fontname, SDL_GetError() );
+    fprintf(stderr, "Error: Can not init font graphics!\n");
     return -1;
   }
 
+  /* Set color palette of the font graphics: */
+  SDL_SetColors(pSmallFontGfx, blackWhiteColors, 0, 2);
+  SDL_SetColors(pBigFontGfx, blackWhiteColors, 0, 2);
+
+  /* Set font color 0 as transparent: */
+  SDL_SetColorKey(pSmallFontGfx, (SDL_SRCCOLORKEY|SDL_RLEACCEL), 0);
+  SDL_SetColorKey(pBigFontGfx, (SDL_SRCCOLORKEY|SDL_RLEACCEL), 0);
+
   return 0;
 }
 
@@ -49,17 +102,18 @@ int SDLGui_Init()
 /*
   Uninitialize the GUI.
 */
-int SDLGui_UnInit()
+int SDLGui_UnInit(void)
 {
-  if(stdfontgfx)
+  if (pSmallFontGfx)
   {
-    SDL_FreeSurface(stdfontgfx);
-    stdfontgfx = NULL;
+    SDL_FreeSurface(pSmallFontGfx);
+    pSmallFontGfx = NULL;
   }
-  if(fontgfx)
+
+  if (pBigFontGfx)
   {
-    SDL_FreeSurface(fontgfx);
-    fontgfx = NULL;
+    SDL_FreeSurface(pBigFontGfx);
+    pBigFontGfx = NULL;
   }
 
   return 0;
@@ -70,31 +124,24 @@ int SDLGui_UnInit()
 /*
   Prepare the font to suit the actual resolution.
 */
-int SDLGui_PrepareFont()
+int SDLGui_PrepareFont(void)
 {
-/* FIXME: Freeing the old font gfx does sometimes crash with a SEGFAULT
-  if(fontgfx)
+  /* Decide which font to use - small or big one: */
+  if (sdlscrn->w >= 640 && sdlscrn->h >= 400 && pBigFontGfx != NULL)
   {
-    SDL_FreeSurface(fontgfx);
-    fontgfx = NULL;
+    fontgfx = pBigFontGfx;
   }
-*/
-
-  if( stdfontgfx == NULL )
+  else
   {
-    fprintf(stderr, "Error: The font has not been loaded!\n");
+    fontgfx = pSmallFontGfx;
+  }
+
+  if (fontgfx == NULL)
+  {
+    fprintf(stderr, "Error: A problem with the font occured!\n");
     return -1;
   }
 
-  /* Convert the font graphics to the actual screen format */
-  fontgfx = SDL_DisplayFormat(stdfontgfx);
-  if( fontgfx == NULL )
-  {
-    fprintf(stderr, "Could not convert font:\n %s\n", SDL_GetError() );
-    return -1;
-  }
-  /* Set transparent pixel */
-  SDL_SetColorKey(fontgfx, (SDL_SRCCOLORKEY|SDL_RLEACCEL), SDL_MapRGB(fontgfx->format,255,255,255));
   /* Get the font width and height: */
   fontwidth = fontgfx->w/16;
   fontheight = fontgfx->h/16;
@@ -120,7 +167,7 @@ void SDLGui_CenterDlg(SGOBJ *dlg)
 /*
   Draw a text string.
 */
-void SDLGui_Text(int x, int y, const char *txt)
+static void SDLGui_Text(int x, int y, const char *txt)
 {
   int i;
   char c;
@@ -142,7 +189,7 @@ void SDLGui_Text(int x, int y, const char *txt)
 /*
   Draw a dialog text object.
 */
-void SDLGui_DrawText(SGOBJ *tdlg, int objnum)
+static void SDLGui_DrawText(SGOBJ *tdlg, int objnum)
 {
   int x, y;
   x = (tdlg[0].x+tdlg[objnum].x)*fontwidth;
@@ -155,7 +202,7 @@ void SDLGui_DrawText(SGOBJ *tdlg, int objnum)
 /*
   Draw a edit field object.
 */
-void SDLGui_DrawEditField(SGOBJ *edlg, int objnum)
+static void SDLGui_DrawEditField(SGOBJ *edlg, int objnum)
 {
   int x, y;
   SDL_Rect rect;
@@ -164,7 +211,7 @@ void SDLGui_DrawEditField(SGOBJ *edlg, int objnum)
   y = (edlg[0].y+edlg[objnum].y)*fontheight;
   SDLGui_Text(x, y, edlg[objnum].txt);
 
-  rect.x = x;    rect.y = y + edlg[objnum].h * fontwidth;
+  rect.x = x;    rect.y = y + edlg[objnum].h * fontheight;
   rect.w = edlg[objnum].w * fontwidth;    rect.h = 1;
   SDL_FillRect(sdlscrn, &rect, SDL_MapRGB(sdlscrn->format,160,160,160));
 }
@@ -174,10 +221,10 @@ void SDLGui_DrawEditField(SGOBJ *edlg, int objnum)
 /*
   Draw a dialog box object.
 */
-void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
+static void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
 {
   SDL_Rect rect;
-  int x, y, w, h;
+  int x, y, w, h, offset;
   Uint32 grey = SDL_MapRGB(sdlscrn->format,192,192,192);
   Uint32 upleftc, downrightc;
 
@@ -202,23 +249,34 @@ void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
     downrightc = SDL_MapRGB(sdlscrn->format,128,128,128);
   }
 
+  /* The root box should be bigger than the screen, so we disable the offset there: */
+  if (objnum != 0)
+    offset = 1;
+  else
+    offset = 0;
+
+  /* Draw background: */
   rect.x = x;  rect.y = y;
   rect.w = w;  rect.h = h;
   SDL_FillRect(sdlscrn, &rect, grey);
 
-  rect.x = x;  rect.y = y-1;
+  /* Draw upper border: */
+  rect.x = x;  rect.y = y - offset;
   rect.w = w;  rect.h = 1;
   SDL_FillRect(sdlscrn, &rect, upleftc);
 
-  rect.x = x-1;  rect.y = y;
+  /* Draw left border: */
+  rect.x = x - offset;  rect.y = y;
   rect.w = 1;  rect.h = h;
   SDL_FillRect(sdlscrn, &rect, upleftc);
 
-  rect.x = x;  rect.y = y+h;
+  /* Draw bottom border: */
+  rect.x = x;  rect.y = y + h - 1 + offset;
   rect.w = w;  rect.h = 1;
   SDL_FillRect(sdlscrn, &rect, downrightc);
 
-  rect.x = x+w;  rect.y = y;
+  /* Draw right border: */
+  rect.x = x + w - 1 + offset;  rect.y = y;
   rect.w = 1;  rect.h = h;
   SDL_FillRect(sdlscrn, &rect, downrightc);
 }
@@ -228,7 +286,7 @@ void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
 /*
   Draw a normal button.
 */
-void SDLGui_DrawButton(SGOBJ *bdlg, int objnum)
+static void SDLGui_DrawButton(SGOBJ *bdlg, int objnum)
 {
   int x,y;
 
@@ -250,7 +308,7 @@ void SDLGui_DrawButton(SGOBJ *bdlg, int objnum)
 /*
   Draw a dialog radio button object.
 */
-void SDLGui_DrawRadioButton(SGOBJ *rdlg, int objnum)
+static void SDLGui_DrawRadioButton(SGOBJ *rdlg, int objnum)
 {
   char str[80];
   int x, y;
@@ -273,7 +331,7 @@ void SDLGui_DrawRadioButton(SGOBJ *rdlg, int objnum)
 /*
   Draw a dialog check box object.
 */
-void SDLGui_DrawCheckBox(SGOBJ *cdlg, int objnum)
+static void SDLGui_DrawCheckBox(SGOBJ *cdlg, int objnum)
 {
   char str[80];
   int x, y;
@@ -296,7 +354,7 @@ void SDLGui_DrawCheckBox(SGOBJ *cdlg, int objnum)
 /*
   Draw a dialog popup button object.
 */
-void SDLGui_DrawPopupButton(SGOBJ *pdlg, int objnum)
+static void SDLGui_DrawPopupButton(SGOBJ *pdlg, int objnum)
 {
   int x, y, w, h;
   const char *downstr = "\x02";
@@ -319,7 +377,7 @@ void SDLGui_DrawPopupButton(SGOBJ *pdlg, int objnum)
   NOTE: The dlg[objnum].txt must point to an an array that is big enough
   for dlg[objnum].w characters!
 */
-void SDLGui_EditField(SGOBJ *dlg, int objnum)
+static void SDLGui_EditField(SGOBJ *dlg, int objnum)
 {
   int cursorPos;                        /* Position of the cursor in the edit field */
   int blinkState = 0;                   /* Used for cursor blinking */
@@ -437,7 +495,7 @@ void SDLGui_EditField(SGOBJ *dlg, int objnum)
 /*
   Draw a whole dialog.
 */
-void SDLGui_DrawDialog(SGOBJ *dlg)
+static void SDLGui_DrawDialog(SGOBJ *dlg)
 {
   int i;
   for(i=0; dlg[i].type!=-1; i++ )
@@ -475,7 +533,7 @@ void SDLGui_DrawDialog(SGOBJ *dlg)
 /*
   Search an object at a certain position.
 */
-int SDLGui_FindObj(SGOBJ *dlg, int fx, int fy)
+static int SDLGui_FindObj(SGOBJ *dlg, int fx, int fy)
 {
   int len, i;
   int ob = -1;
@@ -515,9 +573,38 @@ int SDLGui_DoDialog(SGOBJ *dlg)
   SDL_Event evnt;
   SDL_Rect rct;
   Uint32 grey;
-
+  SDL_Surface *pBgSurface;
+  SDL_Rect dlgrect, bgrect;
+  
   grey = SDL_MapRGB(sdlscrn->format,192,192,192);
 
+  dlgrect.x = dlg[0].x * fontwidth;
+  dlgrect.y = dlg[0].y * fontheight;
+  dlgrect.w = dlg[0].w * fontwidth;
+  dlgrect.h = dlg[0].h * fontheight;
+
+  bgrect.x = bgrect.y = 0;
+  bgrect.w = dlgrect.w;
+  bgrect.h = dlgrect.h;
+
+  /* Save background */
+  pBgSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, dlgrect.w, dlgrect.h, sdlscrn->format->BitsPerPixel,
+                   sdlscrn->format->Rmask, sdlscrn->format->Gmask, sdlscrn->format->Bmask, sdlscrn->format->Amask);
+  if (sdlscrn->format->palette != NULL)
+  {
+    SDL_SetColors(pBgSurface, sdlscrn->format->palette->colors, 0, sdlscrn->format->palette->ncolors-1);
+  }
+
+  if (pBgSurface != NULL)
+  {
+    SDL_BlitSurface(sdlscrn,  &dlgrect, pBgSurface, &bgrect);
+  }
+  else
+  {
+    fprintf(stderr, "SDLGUI_DoDialog: CreateRGBSurface failed: %s\n", SDL_GetError());
+  }
+
+  /* (Re-)draw the dialog */
   SDLGui_DrawDialog(dlg);
 
   /* Is the left mouse button still pressed? Yes -> Handle TOUCHEXIT objects here */
@@ -530,12 +617,12 @@ int SDLGui_DoDialog(SGOBJ *dlg)
     if( b&SDL_BUTTON(1) )
     {
       dlg[obj].state |= SG_SELECTED;
-      return obj;
+      retbutton = obj;
     }
   }
 
   /* The main loop */
-  do
+  while(retbutton==0 && !bQuitProgram)
   {
     if( SDL_WaitEvent(&evnt)==1 )  /* Wait for events */
       switch(evnt.type)
@@ -639,7 +726,13 @@ int SDLGui_DoDialog(SGOBJ *dlg)
           break;
       }
   }
-  while(retbutton==0 && !bQuitProgram);
+
+  /* Restore background */
+  if (pBgSurface)
+  {
+    SDL_BlitSurface(pBgSurface, &bgrect, sdlscrn,  &dlgrect);
+    SDL_FreeSurface(pBgSurface);
+  }
 
   if(bQuitProgram)
     retbutton = -1;
