@@ -14,13 +14,12 @@
   in this game has a bug in it, which corrupts its own registers if more than one byte is queued up. This
   value was found by a test program on a real ST and has correctly emulated the behaviour.
 */
-static char rcsid[] = "Hatari $Id: ikbd.c,v 1.16 2003/05/24 22:12:35 thothy Exp $";
+char IKBD_rcsid[] = "Hatari $Id: ikbd.c,v 1.20 2004/11/14 02:34:31 thothy Exp $";
 
 #include <time.h>
 
 #include "main.h"
 #include "debug.h"
-#include "decode.h"
 #include "gemdos.h"
 #include "ikbd.h"
 #include "int.h"
@@ -35,9 +34,10 @@ static char rcsid[] = "Hatari $Id: ikbd.c,v 1.16 2003/05/24 22:12:35 thothy Exp
 #include "vdi.h"
 
 #define DBL_CLICK_HISTORY  0x07     /* Number of frames since last click to see if need to send one or two clicks */
-#define ACIA_CYCLES    7200         /* Cycles(Multiple of 4) between sent to ACIA from keyboard along serial line - 500Hz/64, (approx' 6920-7200cycles from test program) */
+#define ACIA_CYCLES    7200         /* Cycles (Multiple of 4) between sent to ACIA from keyboard along serial line - 500Hz/64, (approx' 6920-7200cycles from test program) */
 
 #define IKBD_RESET_CYCLES  400000   /* Cycles after RESET before complete */
+#define IKBD_INIT_RESET_CYCLES 3000000  /* Cycles after a cold reset before IKBD starts */
 
 #define ABS_X_ONRESET    0          /* Initial XY for absolute mouse position after RESET command */
 #define ABS_Y_ONRESET    0
@@ -201,7 +201,12 @@ void IKBD_Reset(BOOL bCold)
 {
   /* Reset internal keyboard processor details */
   if (bCold)
+  {
     KeyboardProcessor.bReset = FALSE;
+    if (Int_InterruptActive(INTERRUPT_IKBD_RESETTIMER))
+      Int_RemovePendingInterrupt(INTERRUPT_IKBD_RESETTIMER);
+  }
+
   KeyboardProcessor.MouseMode = AUTOMODE_MOUSEREL;
   KeyboardProcessor.JoystickMode = AUTOMODE_JOYSTICK;
 
@@ -259,7 +264,7 @@ void IKBD_MemorySnapShot_Capture(BOOL bSave)
 /*
   Calculate out 'delta' that mouse has moved by each frame, and add this to our internal keyboard position
 */
-void IKBD_UpdateInternalMousePosition(void)
+static void IKBD_UpdateInternalMousePosition(void)
 {
 
   KeyboardProcessor.Mouse.DeltaX = KeyboardProcessor.Mouse.dx;
@@ -287,10 +292,10 @@ void IKBD_UpdateInternalMousePosition(void)
 /*-----------------------------------------------------------------------*/
 /*
   When running in maximum speed the emulation will not see 'double-clicks' of the mouse
-  as it is running so fast. In this case, we check for a Windows double-click and pass
+  as it is running so fast. In this case, we check for a double-click and pass
   the 'up'/'down' messages in emulation time to simulate the double-click effect!
 */
-void IKBD_CheckForDoubleClicks(void)
+static void IKBD_CheckForDoubleClicks(void)
 {
   /*
     Things get a little complicated when running max speed as a normal
@@ -361,7 +366,7 @@ void IKBD_CheckForDoubleClicks(void)
 /*
   Convert button to BOOL value
 */
-BOOL IKBD_ButtonBool(int Button)
+static BOOL IKBD_ButtonBool(int Button)
 {
   /* Button pressed? */
   if (Button)
@@ -374,7 +379,7 @@ BOOL IKBD_ButtonBool(int Button)
 /*
   Return TRUE if buttons match, use this as buttons are a mask and not BOOLean
 */
-BOOL IKBD_ButtonsEqual(int Button1,int Button2)
+static BOOL IKBD_ButtonsEqual(int Button1,int Button2)
 {
   /* Return BOOL compare */
   return(IKBD_ButtonBool(Button1)==IKBD_ButtonBool(Button2));
@@ -386,7 +391,7 @@ BOOL IKBD_ButtonsEqual(int Button1,int Button2)
   According to if the mouse if enabled or not the joystick 1 fire button/right mouse button
   will become the same button, ie pressing one will also press the other and vise-versa
 */
-void IKBD_DuplicateMouseFireButtons(void)
+static void IKBD_DuplicateMouseFireButtons(void)
 {
   /* Don't duplicate fire button when program tries to use both! */
   if(bBothMouseAndJoy)  return;
@@ -420,7 +425,7 @@ void IKBD_DuplicateMouseFireButtons(void)
 /*
   Send 'relative' mouse position
 */
-void IKBD_SendRelMousePacket(void)
+static void IKBD_SendRelMousePacket(void)
 {
   int ByteRelX,ByteRelY;
   unsigned char Header;
@@ -463,7 +468,7 @@ void IKBD_SendRelMousePacket(void)
 /*
   Send 'joysticks' bit masks
 */
-void IKBD_SelAutoJoysticks(void)
+static void IKBD_SelAutoJoysticks(void)
 {
   unsigned char JoyData;
 
@@ -491,7 +496,7 @@ void IKBD_SelAutoJoysticks(void)
   Send packets which are generated from the mouse action settings
   If relative mode is on, still generate these packets
 */
-void IKBD_SendOnMouseAction(void)
+static void IKBD_SendOnMouseAction(void)
 {
   BOOL bReportPosition = FALSE;
 
@@ -562,13 +567,14 @@ void IKBD_SendOnMouseAction(void)
 /*
   Send mouse movements as cursor keys
 */
-void IKBD_SendCursorMousePacket(void)
+static void IKBD_SendCursorMousePacket(void)
 {
   int i=0;
 
   /* Run each 'Delta' as cursor presses */
-  /* Limit to '10' loops as Windows cursor is a VERY poor quality. Eg, a single mouse movement */
-  /* on and ST gives delta's of '1', mostly, but Windows goes as high as 20+! */
+  /* Limit to '10' loops as host mouse cursor might have a VERY poor quality. */
+  /* Eg, a single mouse movement on and ST gives delta's of '1', mostly, */
+  /* but host mouse might go as high as 20+! */
   while ( (i<10) && ((KeyboardProcessor.Mouse.DeltaX!=0) || (KeyboardProcessor.Mouse.DeltaY!=0)
    || (!IKBD_ButtonsEqual(Keyboard.bOldLButtonDown,Keyboard.bLButtonDown)) || (!IKBD_ButtonsEqual(Keyboard.bOldRButtonDown,Keyboard.bRButtonDown))) ) {
     /* Left? */
@@ -622,10 +628,6 @@ void IKBD_SendCursorMousePacket(void)
 */
 void IKBD_SendAutoKeyboardCommands(void)
 {
-  /* Ignore anything until we've redirected our GEM handlers */
-  if (!bInitGemDOS)
-    return;
-
   /* Don't do anything until processor is first reset */
   if (!KeyboardProcessor.bReset)
     return;
@@ -686,7 +688,7 @@ void IKBD_SendAutoKeyboardCommands(void)
   actually turned back on! (A number of games do this so can get mouse and joystick
   packets at the same time)
 */
-void IKBD_CheckResetDisableBug(void)
+static void IKBD_CheckResetDisableBug(void)
 {
   /* Have disabled BOTH mouse and joystick? */
   if (bMouseDisabled && bJoystickDisabled)
@@ -718,6 +720,9 @@ void IKBD_InterruptHandler_ResetTimer(void)
   /* Remove this interrupt from list and re-order */
   Int_AcknowledgeInterrupt();
 
+  /* Turn processor on; can now process commands */
+  KeyboardProcessor.bReset = TRUE;
+
   /* Critical timer is over */
   bDuringResetCriticalTime = FALSE;
 }
@@ -756,11 +761,11 @@ void IKBD_Cmd_NullFunction(void)
 void IKBD_Cmd_Reset(void)
 {
   /* Check for error series of bytes, eg 0x80,0x01 */
-  if (Keyboard.InputBuffer[1]==0x01) {
+  if (Keyboard.InputBuffer[1] == 0x01)
+  {
 #ifdef DEBUG_OUTPUT_IKBD
     Debug_IKBD("KEYBOARD ON\n");
 #endif
-    KeyboardProcessor.bReset = TRUE;      /* Turn processor on; can now process commands */
 
     /* Set defaults */
     KeyboardProcessor.MouseMode = AUTOMODE_MOUSEREL;
@@ -771,11 +776,15 @@ void IKBD_Cmd_Reset(void)
 
     IKBD_AddKeyToKeyboardBuffer(0xF0);    /* Assume OK, return correct code */
 
-    /* Start timer - some commands are send during this time they may be ignored(see real ST!) */
-    Int_AddRelativeInterrupt(IKBD_RESET_CYCLES,INTERRUPT_IKBD_RESETTIMER);
+    /* Start timer - some commands are send during this time they may be ignored (see real ST!) */
+    if (!KeyboardProcessor.bReset)
+      Int_AddRelativeInterrupt(IKBD_INIT_RESET_CYCLES, INTERRUPT_IKBD_RESETTIMER);
+    else
+      Int_AddRelativeInterrupt(IKBD_RESET_CYCLES, INTERRUPT_IKBD_RESETTIMER);
+
     /* Set this 'critical' flag, gets reset when timer expires */
-    bMouseDisabled = bJoystickDisabled = FALSE;
     bDuringResetCriticalTime = TRUE;
+    bMouseDisabled = bJoystickDisabled = FALSE;
     bBothMouseAndJoy = FALSE;
   }
   /* else if not 0x80,0x01 just ignore */
@@ -1416,7 +1425,7 @@ void IKBD_InterruptHandler_ACIA(void)
   MFP_GPIP &= ~0x10;
 
   /* Acknowledge in MFP circuit, pass bit,enable,pending */
-  MFP_InputOnChannel(MFP_KEYBOARD_BIT,MFP_IERB,&MFP_IPRB);
+  MFP_InputOnChannel(MFP_ACIA_BIT, MFP_IERB, &MFP_IPRB);
 
   /* Clear flag so can allow another byte to be sent along serial line */
   bByteInTransitToACIA = FALSE;
@@ -1470,13 +1479,10 @@ void IKBD_AddKeyToKeyboardBuffer(unsigned char Data)
 
 /*-----------------------------------------------------------------------*/
 /*
-  When press/release key under Windows, execute this function
+  When press/release key under host OS, execute this function.
 */
 void IKBD_PressSTKey(unsigned char ScanCode,BOOL bPress)
 {
-  /* Ignore anything until we've redirected our GEM handlers */
-  if (!bInitGemDOS)
-    return;
   if (!bPress)
     ScanCode |= 0x80;    /* Set top bit if released key */
   IKBD_AddKeyToKeyboardBuffer(ScanCode);  /* And send to keyboard processor */
