@@ -12,12 +12,6 @@
 enum
 {
   INTERRUPT_NULL,
-
-#ifdef USE_DEBUGGER
-  INTERRUPT_DEBUGGER,
-  INTERRUPT_SINGLESTEP,
-#endif  /*USE_DEBUGGER*/
-
   INTERRUPT_VIDEO_VBL,
   INTERRUPT_VIDEO_HBL,
   INTERRUPT_VIDEO_ENDLINE,
