@@ -10,27 +10,10 @@
 
 typedef int BOOL;
 
-#define PROG_NAME      "Hatari v0.45" /* Name, version for window title */
-#define PROG_VERSION        "v0.45"
-#define VERSION_STRING      "0.45 "   /* Always 6 bytes (inc' NULL) */
-#define VERSION_STRING_SIZE    6      /* Size of above (inc' NULL) */
+#define PROG_NAME      "Hatari v0.60" /* Name, version for window title */
+#define PROG_VERSION   "v0.60"
 
-//#define TOTALLY_FINAL_VERSION       /* Web release version... */
-
-#define FINAL_VERSION                 /* Full-speed non-debug version for release */
-//#define DEBUG_TO_FILE               /* Use debug.txt files */
-
-#ifndef FINAL_VERSION
-  #define USE_DEBUGGER                /* Debugger version(non-release) */
-  #define DEBUG_TO_FILE               /* Use debug.txt files */
-#endif
-
-#ifdef TOTALLY_FINAL_VERSION
-  #undef DEBUG_TO_FILE                /* Don't use debug files for final release */
-#endif
-
-#define MAX_FILENAME_LENGTH  256
-#define MAX_STRING_LENGTH  512
+/*#define DEBUG_TO_FILE*/             /* Use debug.txt files */
 
 
 #include <stdio.h>
@@ -42,6 +25,8 @@ typedef int BOOL;
 #include <SDL_types.h>
 
 
+#define MAX_STRING_LENGTH  512
+
 #ifndef FALSE
 #define FALSE 0
 #define TRUE (!0)
@@ -72,7 +57,7 @@ enum {
   REG_A4,
   REG_A5,
   REG_A6,
-  REG_A7,    /* ..A7(also SP) */
+  REG_A7,    /* ..A7 (also SP) */
 };
 
 /* 68000 Condition code's */
@@ -171,13 +156,10 @@ enum {
 #define CYCLES_PER_SEC      (CYCLES_PER_FRAME*50) /* Cycles per second */
 #define CYCLES_ENDLINE      (64+320+88+40)        /* DE(Display Enable) */
 #define CYCLES_HBL          (CYCLES_PER_LINE+96)  /* Cycles for first HBL - very inaccurate on ST */
-#define CYCLES_DEBUGGER     3000        /* Check debugger every 'x' cycles */
 
 /* Illegal Opcode used to help emulation. eg. free entries are 8 to 15 inc' */
 #define  GEMDOS_OPCODE        8  /* Free op-code to intercept GemDOS trap */
-#define  RUNOLDGEMDOS_OPCODE  9  /* Free op-code to set PC to old GemDOS vector(if doesn't need to intercept) */
 #define  SYSINIT_OPCODE      10  /* Free op-code to initialize system (connected drives etc.) */
-#define  TIMERD_OPCODE       11  /* Free op-code to prevent Timer D starting in GemDOS */
 #define  VDI_OPCODE          12  /* Free op-code to call VDI handlers AFTER Trap#2 */
 
 
@@ -187,14 +169,16 @@ enum {
 extern BOOL bQuitProgram;
 extern BOOL bEnableDebug;
 extern BOOL bEmulationActive;
-extern char szBootDiscImage[MAX_FILENAME_LENGTH];
-extern char szWorkingDir[MAX_FILENAME_LENGTH];
+extern char szBootDiscImage[FILENAME_MAX];
+extern char szWorkingDir[FILENAME_MAX];
+
 
 extern void Main_MemorySnapShot_Capture(BOOL bSave);
 extern void Main_SysError(char *Error,char *Title);
-extern int Main_Message(char *lpText, char *lpCaption /*, unsigned int uType*/);
+extern int Main_Message(char *pText, char *pCaption);
 extern void Main_PauseEmulation(void);
 extern void Main_UnPauseEmulation(void);
-extern void Main_EventHandler();
+extern void Main_WarpMouse(int x, int y);
+extern void Main_EventHandler(void);
 
 #endif /* ifndef HATARI_MAIN_H */
