@@ -45,18 +45,18 @@ typedef struct
 {
   int type;             /* What type of object */
   int flags;            /* Object flags */
-  int state;		/* Object state */
+  int state;            /* Object state */
   int x, y;             /* The offset to the upper left corner */
   int w, h;             /* Width and height */
   char *txt;            /* Text string */
 }  SGOBJ;
 
 
-int SDLGui_Init(void);
-int SDLGui_UnInit(void);
-int SDLGui_DoDialog(SGOBJ *dlg);
-int SDLGui_PrepareFont(void);
-void SDLGui_CenterDlg(SGOBJ *dlg);
-int SDLGui_FileSelect(char *path_and_name, char *zip_path, BOOL bAllowNew);
+extern int SDLGui_Init(void);
+extern int SDLGui_UnInit(void);
+extern int SDLGui_PrepareFont(void);
+extern int SDLGui_DoDialog(SGOBJ *dlg);
+extern void SDLGui_CenterDlg(SGOBJ *dlg);
+extern int SDLGui_FileSelect(char *path_and_name, char *zip_path, BOOL bAllowNew);
 
 #endif
