@@ -22,13 +22,12 @@
   testing for addressing into 'no-mans-land' which are parts of the hardware map which are not valid on a
   standard STfm.
 */
-char Intercept_rcsid[] = "Hatari $Id: intercept.c,v 1.16 2003/10/07 20:57:43 thothy Exp $";
+char Intercept_rcsid[] = "Hatari $Id: intercept.c,v 1.24 2004/07/25 13:50:30 thothy Exp $";
 
 #include <SDL_types.h>
 
 #include "main.h"
 #include "debug.h"
-#include "decode.h"
 #include "dialog.h"
 #include "fdc.h"
 #include "int.h"
@@ -40,6 +39,7 @@ char Intercept_rcsid[] = "Hatari $Id: intercept.c,v 1.16 2003/10/07 20:57:43 tho
 #include "midi.h"
 #include "psg.h"
 #include "rtc.h"
+#include "rs232.h"
 #include "screen.h"
 #include "spec512.h"
 #include "stMemory.h"
@@ -53,7 +53,7 @@ char Intercept_rcsid[] = "Hatari $Id: intercept.c,v 1.16 2003/10/07 20:57:43 tho
 
 
 /* A dummy function that does nothing at all... */
-void Intercept_WriteNothing(void)
+static void Intercept_WriteNothing(void)
 {
   /* Nothing... */
 }
@@ -125,6 +125,12 @@ INTERCEPT_ACCESS_FUNC InterceptAccessFuncs[] =
   { 0xfffa23,SIZE_BYTE,Intercept_TimerCData_ReadByte,Intercept_TimerCData_WriteByte },    /* INTERCEPT_TIMERC_DATA */
   { 0xfffa25,SIZE_BYTE,Intercept_TimerDData_ReadByte,Intercept_TimerDData_WriteByte },    /* INTERCEPT_TIMERD_DATA */
 
+  { 0xfffa27,SIZE_BYTE,RS232_SCR_ReadByte,RS232_SCR_WriteByte },    /* Sync character register */
+  { 0xfffa29,SIZE_BYTE,RS232_UCR_ReadByte,RS232_UCR_WriteByte },    /* USART control register */
+  { 0xfffa2b,SIZE_BYTE,RS232_RSR_ReadByte,RS232_RSR_WriteByte },    /* Receiver status register */
+  { 0xfffa2d,SIZE_BYTE,RS232_TSR_ReadByte,RS232_TSR_WriteByte },    /* Transmitter status register */
+  { 0xfffa2f,SIZE_BYTE,RS232_UDR_ReadByte,RS232_UDR_WriteByte },    /* USART data register */
+
   { 0xfffc00,SIZE_BYTE,Intercept_KeyboardControl_ReadByte,Intercept_KeyboardControl_WriteByte }, /* INTERCEPT_KEYBOARDCONTROL */
   { 0xfffc02,SIZE_BYTE,Intercept_KeyboardData_ReadByte,Intercept_KeyboardData_WriteByte },   /* INTERCEPT_KEYBOARDDATA */
   { 0xfffc04,SIZE_BYTE,Intercept_MidiControl_ReadByte,Intercept_MidiControl_WriteByte },  /* INTERCEPT_MIDICONTROL */
@@ -160,6 +166,8 @@ unsigned long noMansLandWorkspace[2] = { (unsigned long)Intercept_NoMansLand_Rea
 unsigned long noMansLandWorkspace[1] = { 0L };
 #endif
 
+static int nTimerDFakeValue;                  /* Faked Timer-D data register for the Timer-D patch */
+
 
 /*-----------------------------------------------------------------------*/
 /*
@@ -219,8 +227,7 @@ void Intercept_UnInit(void)
 
 void Intercept_CreateTable(unsigned long *pInterceptTable[],int Span,int ReadWrite)
 {
-  unsigned int Address, LowAddress, HiAddress;
-  int i;
+  unsigned int Address, LowAddress, HiAddress, i;
 
   /* Scan each hardware address */
   for(Address=0xff8000; Address<=0xffffff; Address++)
@@ -278,7 +285,7 @@ void Intercept_EnableBlitter(BOOL enableFlag)
   Check list of handlers to see if address needs to be intercepted and call
    routines.
 */
-void Intercept_ScanHandlers(unsigned long *the_func)
+static void Intercept_ScanHandlers(unsigned long *the_func)
 {
  if( the_func )
   while( *the_func )      /* Do we have any routines to run for this address? */
@@ -313,7 +320,7 @@ uae_u32 Intercept_ReadByte(uaecptr addr)
   if(addr < 0x00ff8000)
   {
     /* invalid memory addressing --> bus error */
-    M68000_BusError(addr);
+    M68000_BusError(addr, 1);
     return 0;
   }
 
@@ -338,7 +345,7 @@ uae_u32 Intercept_ReadWord(uaecptr addr)
   if(addr < 0x00ff8000)
   {
     /* invalid memory addressing --> bus error */
-    M68000_BusError(addr);
+    M68000_BusError(addr, 1);
     return 0;
   }
 
@@ -363,7 +370,7 @@ uae_u32 Intercept_ReadLong(uaecptr addr)
   if(addr < 0x00ff8000)
   {
     /* invalid memory addressing --> bus error */
-    M68000_BusError(addr);
+    M68000_BusError(addr, 1);
     return 0;
   }
 
@@ -385,7 +392,7 @@ void Intercept_WriteByte(uaecptr addr, uae_u32 val)
   if(addr < 0x00ff8000)
   {
     /* invalid memory addressing --> bus error */
-    M68000_BusError(addr);
+    M68000_BusError(addr, 0);
     return;
   }
 
@@ -409,7 +416,7 @@ void Intercept_WriteWord(uaecptr addr, uae_u32 val)
   if(addr < 0x00ff8000)
   {
     /* invalid memory addressing --> bus error */
-    M68000_BusError(addr);
+    M68000_BusError(addr, 0);
     return;
   }
 
@@ -433,7 +440,7 @@ void Intercept_WriteLong(uaecptr addr, uae_u32 val)
   if(addr < 0x00ff8000)
   {
     /* invalid memory addressing --> bus error */
-    M68000_BusError(addr);
+    M68000_BusError(addr, 0);
     return;
   }
 
@@ -604,13 +611,15 @@ void Intercept_DmaStatus_ReadWord(void)
 /* INTERCEPT_PSG_REGISTER(0xff8800 byte) */
 void Intercept_PSGRegister_ReadByte(void)
 {
- STRam[0xff8800] = PSG_ReadSelectRegister();
+  STRam[0xff8800] = PSG_ReadSelectRegister();
+  M68000_WaitState();
 }
 
 /* INTERCEPT_PSG_DATA(0xff8802 byte) */
 void Intercept_PSGData_ReadByte(void)
 {
- STRam[0xff8802] = PSG_ReadDataRegister();
+  STRam[0xff8802] = PSG_ReadDataRegister();
+  M68000_WaitState();
 }
 
 /* INTERCEPT_MICROWIREDATA(0xff8922 word) */
@@ -735,18 +744,21 @@ void Intercept_TimerCData_ReadByte(void)
  STRam[0xfffa23] = MFP_TC_MAINCOUNTER;
 }
 
-static int timerd_tos_value;
-
 /* INTERCEPT_TIMERD_DATA(0xfffa25 byte) */
 void Intercept_TimerDData_ReadByte(void)
 {
- int pc = m68k_getpc();
- if (pc >= TosAddress && pc <= TosAddress + TosSize) {
-   STRam[0xfffa25] = timerd_tos_value; // trick the tos to believe it was changed
- } else {
-   MFP_ReadTimerD();        /* Stores result in 'MFP_TD_MAINCOUNTER' */
-   STRam[0xfffa25] = MFP_TD_MAINCOUNTER;
- }
+  Uint32 pc = m68k_getpc();
+
+  if (ConfigureParams.System.bPatchTimerD && pc >= TosAddress && pc <= TosAddress + TosSize)
+  {
+    /* Trick the tos to believe it was changed: */
+    STRam[0xfffa25] = nTimerDFakeValue;
+  }
+  else
+  {
+    MFP_ReadTimerD();        /* Stores result in 'MFP_TD_MAINCOUNTER' */
+    STRam[0xfffa25] = MFP_TD_MAINCOUNTER;
+  }
 }
 
 /* INTERCEPT_KEYBOARDCONTROL(0xfffc00 byte) */
@@ -867,7 +879,7 @@ void Intercept_LineWidth_WriteByte(void)
   /* Nothing... */
 }
 
-void Intercept_Colour_WriteWord(unsigned long addr)
+static void Intercept_Colour_WriteWord(Uint32 addr)
 {
  if( !bUseHighRes )                                 /* Don't store if hi-res */
   {
@@ -1007,13 +1019,15 @@ void Intercept_DmaStatus_WriteWord(void)
 /* INTERCEPT_PSG_REGISTER(0xff8800 byte) */
 void Intercept_PSGRegister_WriteByte(void)
 {
- PSG_WriteSelectRegister( STRam[0xff8800] );
+  PSG_WriteSelectRegister( STRam[0xff8800] );
+  M68000_WaitState();
 }
 
 /* INTERCEPT_PSG_DATA(0xff8802 byte) */
 void Intercept_PSGData_WriteByte(void)
 {
- PSG_WriteDataRegister( STRam[0xff8802] );
+  PSG_WriteDataRegister( STRam[0xff8802] );
+  M68000_WaitState();
 }
 
 /* INTERCEPT_MICROWIREDATA(0xff8922 word) */
@@ -1138,19 +1152,42 @@ void Intercept_TimerBCtrl_WriteByte(void)
 /* INTERCEPT_TIMERCD_CTRL(0xfffa1d byte) */
 void Intercept_TimerCDCtrl_WriteByte(void)
 {
- unsigned short old_tcdcr;
- int pc = m68k_getpc();
+  unsigned short old_tcdcr;
 
- old_tcdcr = MFP_TCDCR;             /* Remember old control state */
- MFP_TCDCR = STRam[0xfffa1d];       /* Store new one */
- if( (MFP_TCDCR^old_tcdcr)&0x70 )   /* Check if Timer C control changed */
-   MFP_StartTimerC();               /* Reset timers if need to */
- if( (MFP_TCDCR^old_tcdcr)&0x07 ){   /* Check if Timer D control changed */
-   if (pc >= TosAddress && pc <= TosAddress + TosSize) {
-     MFP_TCDCR = STRam[0xfffa1d] = (STRam[0xfffa1d] & 0xf0) | 7; // slow down timer d if set from tos
-   }
-   MFP_StartTimerD();               /* Reset timers if need to */
- }
+  old_tcdcr = MFP_TCDCR;            /* Remember old control state */
+  MFP_TCDCR = STRam[0xfffa1d];      /* Store new one */
+
+  if ((MFP_TCDCR^old_tcdcr) & 0x70) /* Check if Timer C control changed */
+    MFP_StartTimerC();              /* Reset timers if need to */
+
+  if ((MFP_TCDCR^old_tcdcr) & 0x07) /* Check if Timer D control changed */
+  {
+    Uint32 pc = m68k_getpc();
+
+    /* Need to change baud rate of RS232 emulation? */
+    if (ConfigureParams.RS232.bEnableRS232)
+    {
+      RS232_SetBaudRateFromTimerD();
+    }
+
+    if (ConfigureParams.System.bPatchTimerD && !bAppliedTimerDPatch
+        && pc >= TosAddress && pc <= TosAddress + TosSize)
+    {
+      /* Slow down Timer-D if set from TOS for the first time to gain more
+       * desktop performance.
+       * Obviously, we need to emulate all timers correctly but TOS sets up
+       * Timer-D at a very high rate (every couple of instructions). The
+       * interrupt isn't enabled but the emulator still needs to process the
+       * interrupt table and this HALVES our frame rate!!!
+       * Some games actually reference this timer but don't set it up
+       * (eg Paradroid, Speedball I) so we simply intercept the Timer-D setup
+       * code in TOS and fix the numbers with more 'laid-back' values.
+       * This still keeps 100% compatibility */
+      MFP_TCDCR = STRam[0xfffa1d] = (STRam[0xfffa1d] & 0xf0) | 7;
+      bAppliedTimerDPatch = TRUE;
+    }
+    MFP_StartTimerD();              /* Reset timers if need to */
+  }
 }
 
 /* INTERCEPT_TIMERA_DATA(0xfffa1f byte) */
@@ -1188,16 +1225,25 @@ void Intercept_TimerCData_WriteByte(void)
 /* INTERCEPT_TIMERD_DATA(0xfffa25 byte) */
 void Intercept_TimerDData_WriteByte(void)
 {
- int pc = m68k_getpc();
- if (pc >= TosAddress && pc <= TosAddress + TosSize) {
-   timerd_tos_value = STRam[0xfffa25];
-   STRam[0xfffa25] = 0x64; // slow down the useless interrupt from the bios for timer d
- }
+  Uint32 pc = m68k_getpc();
 
- MFP_TDDR = STRam[0xfffa25];        /* Store into data register */
- if( (MFP_TCDCR&0x07)==0 )          /* Now check if timer is running - if so do not set */
+  /* Need to change baud rate of RS232 emulation? */
+  if (ConfigureParams.RS232.bEnableRS232 && (STRam[0xfffa1d] & 0x07))
   {
-   MFP_StartTimerD();               /* Add our interrupt */
+    RS232_SetBaudRateFromTimerD();
+  }
+
+  /* Patch Timer-D for better performance? */
+  if (ConfigureParams.System.bPatchTimerD && pc >= TosAddress && pc <= TosAddress + TosSize)
+  {
+    nTimerDFakeValue = STRam[0xfffa25];
+    STRam[0xfffa25] = 0x64;         /* Slow down the useless Timer-D setup from the bios */
+  }
+
+  MFP_TDDR = STRam[0xfffa25];       /* Store into data register */
+  if ((MFP_TCDCR&0x07) == 0)        /* Now check if timer is running - if so do not set */
+  {
+    MFP_StartTimerD();              /* Add our interrupt */
   }
 }
 
@@ -1300,9 +1346,14 @@ INTERCEPT_ADDRESSRANGE InterceptBusErrors[] =
 /*
   Jump to the BusError handler with the correct bus address
 */
-void Intercept_BusError(void)
+static void Intercept_BusErrorReadAccess(void)
 {
-  M68000_BusError(BusAddressLocation);
+  M68000_BusError(BusAddressLocation, 1);
+}
+
+static void Intercept_BusErrorWriteAccess(void)
+{
+  M68000_BusError(BusAddressLocation, 0);
 }
 
 
@@ -1314,13 +1365,17 @@ void Intercept_BusError(void)
 */
 void Intercept_ModifyTablesForBusErrors(void)
 {
-  unsigned long *pInterceptList;
+  unsigned long *pInterceptListRead, *pInterceptListWrite;
   unsigned int Address;
   int i;
 
   /* Set routine list */
-  pInterceptList = pCurrentInterceptWorkspace;
-  *pCurrentInterceptWorkspace++ = (unsigned long)Intercept_BusError;
+  pInterceptListRead = pCurrentInterceptWorkspace;
+  *pCurrentInterceptWorkspace++ = (unsigned long)Intercept_BusErrorReadAccess;
+  *pCurrentInterceptWorkspace++ = 0L;
+
+  pInterceptListWrite = pCurrentInterceptWorkspace;
+  *pCurrentInterceptWorkspace++ = (unsigned long)Intercept_BusErrorWriteAccess;
   *pCurrentInterceptWorkspace++ = 0L;
 
   /* Set all bus-error entries */
@@ -1332,13 +1387,13 @@ void Intercept_ModifyTablesForBusErrors(void)
     for(Address=InterceptBusErrors[i].Start_Address; Address<InterceptBusErrors[i].End_Address; Address++)
     {
       /* For 'read' */
-      pInterceptReadByteTable[Address-0xff8000] = pInterceptList;
-      pInterceptReadWordTable[Address-0xff8000] = pInterceptList;
-      pInterceptReadLongTable[Address-0xff8000] = pInterceptList;
+      pInterceptReadByteTable[Address-0xff8000] = pInterceptListRead;
+      pInterceptReadWordTable[Address-0xff8000] = pInterceptListRead;
+      pInterceptReadLongTable[Address-0xff8000] = pInterceptListRead;
       /* and 'write' */
-      pInterceptWriteByteTable[Address-0xff8000] = pInterceptList;
-      pInterceptWriteWordTable[Address-0xff8000] = pInterceptList;
-      pInterceptWriteLongTable[Address-0xff8000] = pInterceptList;
+      pInterceptWriteByteTable[Address-0xff8000] = pInterceptListWrite;
+      pInterceptWriteWordTable[Address-0xff8000] = pInterceptListWrite;
+      pInterceptWriteLongTable[Address-0xff8000] = pInterceptListWrite;
     }
 
   }
