@@ -1,12 +1,12 @@
 /*
-  Hatari - main.c
+  Hatari - keymap.c
 
   This file is distributed under the GNU Public License, version 2 or at
   your option any later version. Read the file gpl.txt for details.
 
   Here we process a key press and the remapping of the scancodes.
 */
-static char rcsid[] = "Hatari $Id: keymap.c,v 1.14 2003/08/09 13:58:42 simonsunnyboy Exp $";
+char Keymap_rcsid[] = "Hatari $Id: keymap.c,v 1.16 2004/04/19 08:53:33 thothy Exp $";
 
 #include "main.h"
 #include "debug.h"
@@ -96,6 +96,10 @@ Ins   Del
 */
 
 
+/* Mask of valid shortcut key modifiers */
+#define KEYMAP_SHORTCUTMODS  (KMOD_RALT|KMOD_LMETA|KMOD_RMETA|KMOD_MODE)
+
+
 /* SDL symbolic key to ST scan code mapping table */
 static const char SymbolicKeyToSTScanCode[SDLK_LAST] =
 {
@@ -545,8 +549,6 @@ static char Keymap_GetKeyPadScanCode(SDL_keysym* pKeySym)
 */
 char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
 {
-  static BOOL bScanCodesInitialized = FALSE;
-
   if(pKeySym->sym >= SDLK_LAST)  return -1;  /* Avoid illegal keys */
 
   /* Check for keypad first so we can handle numlock */
@@ -636,7 +638,7 @@ void Keymap_LoadRemapFile(char *pszFileName)
   Scan list of keys to NOT de-bounce when running in maximum speed, eg ALT,SHIFT,CTRL etc...
   Return TRUE if key requires de-bouncing
 */
-BOOL Keymap_DebounceSTKey(char STScanCode)
+static BOOL Keymap_DebounceSTKey(char STScanCode)
 {
   int i=0;
 
@@ -669,7 +671,6 @@ void Keymap_DebounceAllKeys(void)
 {
   unsigned int key;
   char STScanCode;
-  BOOL bUseScanCodes;
   SDL_keysym tmpKeySym;
 
   /* Return if we aren't in maximum speed or have not disabled key repeat */
@@ -713,14 +714,13 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
   BOOL bPreviousKeyState;
   char STScanCode;
   int symkey = sdlkey->sym;
-  int scankey = sdlkey->scancode;
   int modkey = sdlkey->mod;
 
-  /*fprintf(stderr, "keydown: sym=%i scan=%i mod=$%x\n",symkey, scankey, modkey);*/
+  /*fprintf(stderr, "keydown: sym=%i scan=%i mod=$%x\n",symkey, sdlkey->scancode, modkey);*/
 
   /* If using cursor emulation, DON'T send keys to keyboard processor!!! Some games use keyboard as pause! */
   if((ConfigureParams.Joysticks.Joy[0].bCursorEmulation || ConfigureParams.Joysticks.Joy[1].bCursorEmulation)
-     && !(modkey&(KMOD_LSHIFT|KMOD_RSHIFT)))
+     && !(modkey & KMOD_SHIFT))
   {
     if(symkey == SDLK_UP)
       { cursorJoyEmu |= 1; return; }
@@ -735,7 +735,8 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
   }
 
   /* Handle special keys */
-  if(symkey == SDLK_MODE || symkey == SDLK_LMETA || symkey == SDLK_NUMLOCK)
+  if (symkey == SDLK_RALT || symkey == SDLK_LMETA || symkey == SDLK_RMETA
+      || symkey == SDLK_MODE || symkey == SDLK_NUMLOCK)
   {
     /* Ignore modifier keys that aren't passed to the ST */
     return;
@@ -758,11 +759,11 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
   Keyboard.KeyStates[symkey] = TRUE;
 
   /* If pressed short-cut key, retain keypress until safe to execute (start of VBL) */
-  if((modkey&KMOD_MODE) || (modkey&KMOD_LMETA))
+  if (modkey & KEYMAP_SHORTCUTMODS)
   {
     ShortCutKey.Key = symkey;
-    if( modkey&(KMOD_LCTRL|KMOD_RCTRL) )  ShortCutKey.bCtrlPressed = TRUE;
-    if( modkey&(KMOD_LSHIFT|KMOD_RSHIFT) )  ShortCutKey.bShiftPressed = TRUE;
+    if (modkey & KMOD_CTRL)  ShortCutKey.bCtrlPressed = TRUE;
+    if (modkey & KMOD_SHIFT)  ShortCutKey.bShiftPressed = TRUE;
   }
   else
   {
@@ -784,14 +785,13 @@ void Keymap_KeyUp(SDL_keysym *sdlkey)
 {
   char STScanCode;
   int symkey = sdlkey->sym;
-  int scankey = sdlkey->scancode;
   int modkey = sdlkey->mod;
 
-  /*fprintf(stderr, "keyup: sym=%i scan=%i mod=$%x\n",symkey, scankey, modkey);*/
+  /*fprintf(stderr, "keyup: sym=%i scan=%i mod=$%x\n",symkey, sdlkey->scancode, modkey);*/
 
   /* If using cursor emulation, DON'T send keys to keyboard processor!!! Some games use keyboard as pause! */
   if((ConfigureParams.Joysticks.Joy[0].bCursorEmulation || ConfigureParams.Joysticks.Joy[1].bCursorEmulation)
-     && !(modkey&(KMOD_LSHIFT|KMOD_RSHIFT)))
+     && !(modkey & KMOD_SHIFT))
   {
     if(symkey == SDLK_UP)
       { cursorJoyEmu &= ~1; return; }
@@ -806,7 +806,8 @@ void Keymap_KeyUp(SDL_keysym *sdlkey)
   }
 
   /* Handle special keys */
-  if(symkey == SDLK_MODE || symkey == SDLK_LMETA || symkey == SDLK_NUMLOCK)
+  if (symkey == SDLK_RALT || symkey == SDLK_LMETA || symkey == SDLK_RMETA
+      || symkey == SDLK_MODE || symkey == SDLK_NUMLOCK)
   {
     /* Ignore modifier keys that aren't passed to the ST */
     return;
