@@ -8,12 +8,11 @@
   few OpCode's such as Line-F and Line-A. In Hatari it has mainly become a
   wrapper between the WinSTon sources and the UAE CPU code.
 */
-char M68000_rcsid[] = "Hatari $Id: m68000.c,v 1.24 2003/10/25 12:26:39 thothy Exp $";
+char M68000_rcsid[] = "Hatari $Id: m68000.c,v 1.28 2004/06/11 10:04:46 thothy Exp $";
 
 #include "main.h"
 #include "bios.h"
 #include "debug.h"
-#include "decode.h"
 #include "gemdos.h"
 #include "hatari-glue.h"
 #include "int.h"
@@ -26,12 +25,12 @@ char M68000_rcsid[] = "Hatari $Id: m68000.c,v 1.24 2003/10/25 12:26:39 thothy Ex
 #include "xbios.h"
 
 
-short int PendingInterruptFlag;
 void *PendingInterruptFunction;
 short int PendingInterruptCount;
+
 Uint32 BusAddressLocation;       /* Stores the offending address for bus-/address errors */
 Uint32 BusErrorPC;               /* Value of the PC when bus error occurs */
-Uint16 BusErrorOpcode;           /* Opcode of faulting instruction */
+BOOL bBusErrorReadWrite;         /* 0 for write error, 1 for read error */
 
 
 /*-----------------------------------------------------------------------*/
@@ -49,8 +48,6 @@ void M68000_Reset(BOOL bCold)
       Regs[i] = 0;
   }
 
-  PendingInterruptFlag = 0;     /* Clear pending flag */
-
   /* Now directly reset the UAE CPU core: */
   m68k_reset();
 }
@@ -69,7 +66,6 @@ void M68000_MemorySnapShot_Capture(BOOL bSave)
   MemorySnapShot_Store(Regs,sizeof(Regs));
   MemorySnapShot_Store(&STRamEnd,sizeof(STRamEnd));
   MemorySnapShot_Store(&PendingInterruptCount,sizeof(PendingInterruptCount));
-  MemorySnapShot_Store(&PendingInterruptFlag,sizeof(PendingInterruptFlag));
   if (bSave)
   {
     /* Convert function to ID */
@@ -146,9 +142,10 @@ void M68000_MemorySnapShot_Capture(BOOL bSave)
 
 /*-----------------------------------------------------------------------*/
 /*
-  BUSERROR - Access outside valid memory range
+  BUSERROR - Access outside valid memory range.
+  Use bReadWrite = 0 for write errors and bReadWrite = 1 for read errors!
 */
-void M68000_BusError(unsigned long addr)
+void M68000_BusError(unsigned long addr, BOOL bReadWrite)
 {
   /* FIXME: In prefetch mode, m68k_getpc() seems already to point to the next instruction */
   BusErrorPC = m68k_getpc();
@@ -160,7 +157,7 @@ void M68000_BusError(unsigned long addr)
   }
 
   BusAddressLocation = addr;        /* Store for exception frame */
-  BusErrorOpcode = get_word(BusErrorPC);
+  bBusErrorReadWrite = bReadWrite;
   set_special(SPCFLAG_BUSERROR);    /* The exception will be done in newcpu.c */
 }
 
@@ -224,3 +221,13 @@ void M68000_Exception(Uint32 ExceptionVector)
   }
 }
 
+
+/*-----------------------------------------------------------------------*/
+/*
+  There seem to be wait states when a program accesses certain hardware
+  registers on the ST. Use this function to simulate these wait states.
+*/
+void M68000_WaitState(void)
+{
+  set_special(SPCFLAG_EXTRA_CYCLES);
+}
