@@ -6,18 +6,15 @@
 
   Main initialization and event handling routines.
 */
-static char rcsid[] = "Hatari $Id: main.c,v 1.47 2003/09/27 19:28:50 thothy Exp $";
+char Main_rcsid[] = "Hatari $Id: main.c,v 1.66 2004/12/09 21:06:37 thothy Exp $";
 
 #include <time.h>
-#include <signal.h>
-#include <sys/time.h>
 #include <unistd.h>
 
 #include <SDL.h>
 
 #include "main.h"
 #include "configuration.h"
-#include "decode.h"
 #include "dialog.h"
 #include "audio.h"
 #include "debug.h"
@@ -29,18 +26,19 @@ static char rcsid[] = "Hatari $Id: main.c,v 1.47 2003/09/27 19:28:50 thothy Exp
 #include "hdc.h"
 #include "ikbd.h"
 #include "intercept.h"
-#include "reset.h"
 #include "keymap.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "misc.h"
 #include "midi.h"
 #include "printer.h"
+#include "reset.h"
 #include "rs232.h"
 #include "screen.h"
 #include "sdlgui.h"
 #include "shortcut.h"
 #include "sound.h"
+#include "stMemory.h"
 #include "tos.h"
 #include "vdi.h"
 #include "video.h"
@@ -51,15 +49,11 @@ static char rcsid[] = "Hatari $Id: main.c,v 1.47 2003/09/27 19:28:50 thothy Exp
 
 
 BOOL bQuitProgram=FALSE;                  /* Flag to quit program cleanly */
-BOOL bUseFullscreen=FALSE;
 BOOL bEmulationActive=TRUE;               /* Run emulation when started */
-BOOL bAppActive = FALSE;
 BOOL bEnableDebug=FALSE;                  /* Enable debug UI? */
-char szBootDiscImage[MAX_FILENAME_LENGTH] = { "" };
-char szWorkingDir[MAX_FILENAME_LENGTH];   /* Working directory */
-
-unsigned char STRam[16*1024*1024];        /* This is our ST Ram, includes all TOS/hardware areas for ease */
-
+char szBootDiscImage[FILENAME_MAX];
+char szWorkingDir[FILENAME_MAX];          /* Working directory */
+BOOL bIgnoreNextMouseMotion = FALSE;      /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
 
 
 /*-----------------------------------------------------------------------*/
@@ -100,16 +94,14 @@ void Main_SysError(char *Error,char *Title)
 
 /*-----------------------------------------------------------------------*/
 /*
-  Bring up message(handles full-screen as well as Window)
+  Bring up a message dialog.
 */
-int Main_Message(char *lpText, char *lpCaption/*,unsigned int uType*/)
+int Main_Message(char *pText, char *pCaption)
 {
-  int Ret=0;
+  /* Show message: */
+  fprintf(stderr,"%s: %s\n", pCaption, pText);
 
-  /* Show message */
-  fprintf(stderr,"%s: %s\n", lpCaption, lpText);
-
-  return(Ret);
+  return DlgAlert_Notice(pText);
 }
 
 
@@ -135,7 +127,6 @@ void Main_UnPauseEmulation(void)
   if( !bEmulationActive )
   {
     Audio_EnableAudio(ConfigureParams.Sound.bEnableSound);
-    bFullScreenHold = FALSE;      /* Release hold  */
     Screen_SetFullUpdate();       /* Cause full screen update(to clear all) */
 
     bEmulationActive = TRUE;
@@ -143,26 +134,49 @@ void Main_UnPauseEmulation(void)
 }
 
 
+/* ----------------------------------------------------------------------- */
+/*
+  Set mouse pointer to new coordinates and set flag to ignore the mouse event
+  that is generated by SDL_WarpMouse().
+*/
+void Main_WarpMouse(int x, int y)
+{
+  SDL_WarpMouse(x, y);                  /* Set mouse pointer to new position */
+  bIgnoreNextMouseMotion = TRUE;        /* Ignore mouse motion event from SDL_WarpMouse */
+}
+
+
 /* ----------------------------------------------------------------------- */
 /*
   Message handler
   Here we process the SDL events (keyboard, mouse, ...) and map it to
   Atari IKBD events.
 */
-void Main_EventHandler()
+void Main_EventHandler(void)
 {
   SDL_Event event;
 
   if( SDL_PollEvent(&event) )
    switch( event.type )
    {
+
     case SDL_QUIT:
        bQuitProgram = TRUE;
+       set_special(SPCFLAG_BRK);        /* Assure that CPU core shuts down */
        break;
+
     case SDL_MOUSEMOTION:               /* Read/Update internal mouse position */
-       KeyboardProcessor.Mouse.dx += event.motion.xrel;
-       KeyboardProcessor.Mouse.dy += event.motion.yrel;
+       if (bIgnoreNextMouseMotion)
+       {
+         bIgnoreNextMouseMotion = FALSE;
+       }
+       else
+       {
+         KeyboardProcessor.Mouse.dx += event.motion.xrel;
+         KeyboardProcessor.Mouse.dy += event.motion.yrel;
+       }
        break;
+
     case SDL_MOUSEBUTTONDOWN:
        if( event.button.button==SDL_BUTTON_LEFT )
        {
@@ -174,15 +188,18 @@ void Main_EventHandler()
        else if( event.button.button==SDL_BUTTON_MIDDLE )
          Keyboard.LButtonDblClk = 1;    /* Start double-click sequence in emulation time */
        break;
+
     case SDL_MOUSEBUTTONUP:
        if( event.button.button==SDL_BUTTON_LEFT )
          Keyboard.bLButtonDown &= ~BUTTON_MOUSE;
        else if( event.button.button==SDL_BUTTON_RIGHT )
          Keyboard.bRButtonDown &= ~BUTTON_MOUSE;;
        break;
+
     case SDL_KEYDOWN:
        Keymap_KeyDown(&event.key.keysym);
        break;
+
     case SDL_KEYUP:
        Keymap_KeyUp(&event.key.keysym);
        break;
@@ -190,11 +207,48 @@ void Main_EventHandler()
 }
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  Show supported options.
+*/
+static void Main_ShowOptions(void)
+{
+  printf("Usage:\n hatari [options] [disk image name]\n"
+         "Where options are:\n"
+         "  --help or -h          Print this help text and exit.\n"
+         "  --version or -v       Print version number and exit.\n"
+         "  --mono or -m          Start in monochrome mode instead of color.\n"
+         "  --fullscreen or -f    Start emulator in fullscreen mode.\n"
+         "  --window or -w        Start emulator in window mode.\n"
+         "  --joystick or -j      Emulate a ST joystick with the cursor keys.\n"
+         "  --nosound             Disable sound (faster!).\n"
+         "  --printer             Enable printer support (experimental).\n"
+         "  --midi <filename>     Enable midi support and write midi data to <filename>.\n"
+         "  --rs232 <filename>    Use <filename> as the serial port device.\n"
+         "  --frameskip           Skip every second frame (speeds up emulation!).\n"
+         "  --debug or -D         Allow debug interface.\n"
+         "  --harddrive <dir>     Emulate an ST harddrive\n"
+         "     or -d <dir>         (<dir> = root directory).\n"
+         "  --hdimage <imagename> Emulate an ST harddrive with an image.\n"
+         "  --tos <file>          Use TOS image <file>.\n"
+         "  --cartridge <file>    Use ROM cartridge image <file>.\n"
+         "  --cpulevel <x>        Set the CPU type (x => 680x0) (TOS 2.06 only!).\n"
+         "  --compatible          Use a more compatible (but slower) 68000 CPU mode.\n"
+         "  --blitter             Enable blitter emulation (unstable!)\n"
+         "  --vdi                 Use extended VDI resolution\n"
+         "  --memsize <x>         Memory size in MB (x = 0, 1, 2 or 4; 0 for 512kB)\n"
+         "  --configfile <file>   Use <file> instead of ~/.hatari.cfg as configuration\n"
+         "     or -c <file>        file.\n"
+         "  --slowfdc             Slow down FDC emulation (very experimental!).\n"
+        );
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Check for any passed parameters
 */
-void Main_ReadParameters(int argc, char *argv[])
+static void Main_ReadParameters(int argc, char *argv[])
 {
   int i;
 
@@ -205,28 +259,7 @@ void Main_ReadParameters(int argc, char *argv[])
     {
       if (!strcmp(argv[i],"--help") || !strcmp(argv[i],"-h"))
       {
-        printf("Usage:\n hatari [options] [disk image name]\n"
-               "Where options are:\n"
-               "  --help or -h          Print this help text and exit.\n"
-               "  --version or -v       Print version number and exit.\n"
-               "  --mono or -m          Start in monochrome mode instead of color.\n"
-               "  --fullscreen or -f    Try to use fullscreen mode.\n"
-               "  --joystick or -j      Emulate a ST joystick with the cursor keys.\n"
-               "  --nosound             Disable sound (faster!).\n"
-               "  --printer             Enable printer support (experimental).\n"
-               "  --midi <filename>     Enable midi support and write midi data to <filename>.\n"
-               "  --frameskip           Skip every second frame (speeds up emulation!).\n"
-               "  --debug or -D         Allow debug interface.\n"
-               "  --harddrive <dir>     Emulate an ST harddrive\n"
-               "     or -d <dir>         (<dir> = root directory).\n"
-               "  --hdimage <imagename> Emulate an ST harddrive with an image.\n"
-               "  --tos <file>          Use TOS image <file>.\n"
-               "  --cpulevel <x>        Set the CPU type (x => 680x0) (TOS 2.06 only!).\n"
-               "  --compatible          Use a more compatible (but slower) 68000 CPU mode.\n"
-               "  --blitter             Enable blitter emulation (unstable!)\n"
-               "  --vdi                 Use extended VDI resolution\n"
-               "  --memsize <x>         Memory size in MB (x = 0, 1, 2 or 4; 0 for 512kB)\n"
-              );
+        Main_ShowOptions();
         exit(0);
       }
       else if (!strcmp(argv[i],"--version") || !strcmp(argv[i],"-v"))
@@ -243,7 +276,11 @@ void Main_ReadParameters(int argc, char *argv[])
       }
       else if (!strcmp(argv[i],"--fullscreen") || !strcmp(argv[i],"-f"))
       {
-        bUseFullscreen=TRUE;
+        ConfigureParams.Screen.bFullScreen = TRUE;
+      }
+      else if (!strcmp(argv[i],"--window") || !strcmp(argv[i],"-w"))
+      {
+        ConfigureParams.Screen.bFullScreen = FALSE;
       }
       else if (!strcmp(argv[i],"--joystick") || !strcmp(argv[i],"-j"))
       {
@@ -251,9 +288,12 @@ void Main_ReadParameters(int argc, char *argv[])
       }
       else if ( !strcmp(argv[i],"--nosound") )
       {
-        bDisableSound=TRUE;
         ConfigureParams.Sound.bEnableSound = FALSE;
       }
+      else if ( !strcmp(argv[i],"--frameskip") )
+      {
+        ConfigureParams.Screen.bFrameSkip = TRUE;
+      }
       else if ( !strcmp(argv[i],"--printer") )
       {
         /* FIXME: add more commandline configuration for printing */
@@ -265,7 +305,7 @@ void Main_ReadParameters(int argc, char *argv[])
           fprintf(stderr, "Missing argument for --midi\n");
         else
         {
-          if(strlen(argv[i+1]) <= MAX_FILENAME_LENGTH)
+          if (strlen(argv[i+1]) <= sizeof(ConfigureParams.Midi.szMidiOutFileName))
           {
             ConfigureParams.Midi.bEnableMidi = TRUE;
             strcpy(ConfigureParams.Midi.szMidiOutFileName, argv[i+1]);
@@ -274,6 +314,22 @@ void Main_ReadParameters(int argc, char *argv[])
           i += 1;
         }
       }
+      else if (!strcmp(argv[i], "--rs232"))
+      {
+        if(i+1 >= argc)
+          fprintf(stderr, "Missing argument for --rs232\n");
+        else
+        {
+          if (strlen(argv[i+1]) <= sizeof(ConfigureParams.RS232.szOutFileName))
+          {
+            ConfigureParams.RS232.bEnableRS232 = TRUE;
+            strcpy(ConfigureParams.RS232.szOutFileName, argv[i+1]);
+            strcpy(ConfigureParams.RS232.szInFileName, argv[i+1]);
+          }
+          else fprintf(stderr, "RS232 file name too long!\n");
+          i += 1;
+        }
+      }
       else if (!strcmp(argv[i],"--debug") || !strcmp(argv[i],"-D"))
       {
         bEnableDebug=TRUE;
@@ -284,7 +340,7 @@ void Main_ReadParameters(int argc, char *argv[])
           fprintf(stderr, "Missing argument for --hdimage\n");
         else
         {
-          if(strlen(argv[i+1]) <= MAX_FILENAME_LENGTH)
+          if (strlen(argv[i+1]) <= sizeof(ConfigureParams.HardDisc.szHardDiscImage))
           {
             ConfigureParams.HardDisc.bUseHardDiscImage = TRUE;
             strcpy(ConfigureParams.HardDisc.szHardDiscImage, argv[i+1]);
@@ -314,7 +370,14 @@ void Main_ReadParameters(int argc, char *argv[])
         if(i+1>=argc)
           fprintf(stderr,"Missing argument for --tos.\n");
         else
-          strncpy(ConfigureParams.TOSGEM.szTOSImageFileName, argv[++i], MAX_FILENAME_LENGTH);
+          strncpy(ConfigureParams.Rom.szTosImageFileName, argv[++i], sizeof(ConfigureParams.Rom.szTosImageFileName));
+      }
+      else if (!strcmp(argv[i],"--cartridge"))
+      {
+        if(i+1>=argc)
+          fprintf(stderr,"Missing argument for --cartridge.\n");
+        else
+          strncpy(ConfigureParams.Rom.szCartridgeImageFileName, argv[++i], sizeof(ConfigureParams.Rom.szCartridgeImageFileName));
       }
       else if (!strcmp(argv[i],"--cpulevel"))
       {
@@ -338,7 +401,7 @@ void Main_ReadParameters(int argc, char *argv[])
       }
       else if (!strcmp(argv[i], "--vdi"))
       {
-        bUseVDIRes = ConfigureParams.TOSGEM.bUseExtGEMResolutions = TRUE;
+        bUseVDIRes = ConfigureParams.Screen.bUseExtVdiResolutions = TRUE;
       }
       else if (!strcmp(argv[i], "--memsize"))
       {
@@ -356,13 +419,37 @@ void Main_ReadParameters(int argc, char *argv[])
         else  /* Use 1MB as default */
           ConfigureParams.Memory.nMemorySize = MEMORY_SIZE_1Mb;
       }
+      else if (!strcmp(argv[i],"--configfile") || !strcmp(argv[i],"-c"))
+      {
+        if (i+1 >= argc)
+          fprintf(stderr, "Missing argument for --configfile\n");
+        else
+        {
+          if (strlen(argv[i+1]) <= sizeof(sConfigFileName))
+          {
+            strcpy(sConfigFileName, argv[i+1]);
+            Configuration_Load();
+          }
+          else
+            fprintf(stderr, "Config file name too long!\n");
+          i += 1;
+        }
+      }
+      else if (!strcmp(argv[i], "--slowfdc"))
+      {
+        ConfigureParams.System.bSlowFDC = TRUE;
+      }
       else
       {
         /* Possible passed disc image filename, ie starts with character other than '-' */
-        if (argv[i][0]!='-')
-          strcpy(szBootDiscImage,argv[i]);
+        if (argv[i][0] != '-' && strlen(argv[i]) < sizeof(szBootDiscImage)
+            && File_Exists(argv[i]))
+        {
+          strcpy(szBootDiscImage, argv[i]);
+          File_MakeAbsoluteName(szBootDiscImage);
+        }
         else
-          fprintf(stderr,"Illegal parameter: %s\n",argv[i]);
+          fprintf(stderr,"Illegal parameter: %s\n", argv[i]);
       }
     }
   }
@@ -373,7 +460,7 @@ void Main_ReadParameters(int argc, char *argv[])
 /*
   Initialise emulation
 */
-void Main_Init(void)
+static void Main_Init(void)
 {
   /* Init SDL's video subsystem. Note: Audio and joystick subsystems
      will be initialized later (failures there are not fatal). */
@@ -438,10 +525,9 @@ void Main_Init(void)
 /*
   Un-Initialise emulation
 */
-void Main_UnInit(void)
+static void Main_UnInit(void)
 {
   Screen_ReturnFromFullScreen();
-  Floppy_EjectBothDrives();
   Floppy_UnInit();
   HDC_UnInit();
   Midi_UnInit();
@@ -473,7 +559,9 @@ int main(int argc, char *argv[])
   srand( time(NULL) );
 
   /* Get working directory */
-  getcwd(szWorkingDir, MAX_FILENAME_LENGTH);
+  getcwd(szWorkingDir, FILENAME_MAX);
+
+  szBootDiscImage[0] = 0;
 
   /* Create debug files */
   Debug_OpenFiles();
@@ -492,7 +580,7 @@ int main(int argc, char *argv[])
   Main_Init();
 
   /* Switch immediately to fullscreen if user wants to */
-  if( bUseFullscreen )
+  if (ConfigureParams.Screen.bFullScreen)
     Screen_EnterFullScreen();
 
   /* Run emulation */
