@@ -14,11 +14,10 @@
   It shows the main details of the chip's behaviour with regard to interrupts
   and pending/service bits.
 */
-static char rcsid[] = "Hatari $Id: mfp.c,v 1.9 2003/08/09 13:58:42 simonsunnyboy Exp $";
+char MFP_rcsid[] = "Hatari $Id: mfp.c,v 1.15 2004/06/11 10:04:46 thothy Exp $";
 
 #include "main.h"
 #include "debug.h"
-#include "decode.h"
 #include "fdc.h"
 #include "ikbd.h"
 #include "int.h"
@@ -53,7 +52,11 @@ Input -----/             |         ------------------------              |
                          --------------------  --------------------------------------o--- PassVector
 */
 
-#define  IGNORE_FDC_PRIORITY    0xff    /* Set 0xff if FDC has higher priority than Keyboard/Midi, or 0x7f if not - Dragon's Breath will now work! */
+
+/*-----------------------------------------------------------------------*/
+/* Set clock times for each instruction, see '68000 timing' pages for details */
+#define  ROUND_CYCLES_TO4(var)  (((int)(var)+3)&0xfffffffc)
+
 
 /* MFP Registers */
 unsigned char MFP_GPIP;               /* General Purpose Pins */
@@ -72,10 +75,12 @@ unsigned char MFP_TC_MAINCOUNTER;     /* Timer C Main Counter (these are temp's,
 unsigned char MFP_TD_MAINCOUNTER;     /* Timer D Main Counter (as done via interrupts) */
 
 /* CPU clock cycle counts for each timer */
-int TimerAClockCycles=0;
-int TimerBClockCycles=0;
-int TimerCClockCycles=0;
-int TimerDClockCycles=0;
+static int TimerAClockCycles=0;
+static int TimerBClockCycles=0;
+static int TimerCClockCycles=0;
+static int TimerDClockCycles=0;
+
+BOOL bAppliedTimerDPatch;             /* TRUE if the Timer-D patch has been applied */
 
 /*
  Number of CPU cycles for Timer C+D
@@ -83,7 +88,7 @@ int TimerDClockCycles=0;
  Now, Timer C set on a delay of 192($C0) and a preset DIV of 64 is 200Hz
  This makes the table entry 208.66666*192=40064(200Hz)
 */
-float MFPTimerToCPUCycleTable[] = {
+static float MFPTimerToCPUCycleTable[] = {
    0,             /* Timer Stop */
    13.04166667f,  /* Div by 4  */
    32.60416667f,  /* Div by 10 */
@@ -103,11 +108,13 @@ void MFP_Reset(void)
 {
   /* Reset MFP internal variables */
 
+  bAppliedTimerDPatch = FALSE;
+
   /* NOTE  Matthias Arndt <marndt@asmsoftware.de>  9 Aug 2003
    * according to the Atari ST Profibuch, Bit0 of GPIP Data Register
    * is the BUSY signal of the printer port
    * it is SET if no printer is connected or on BUSY
-   * therefor we should assume it to be 0 in Hatari as printer bsuy is
+   * therefor we should assume it to be 0 in Hatari as printer busy is
    * all handled in the Printer submodule
    */
 
@@ -174,7 +181,7 @@ void MFP_MemorySnapShot_Capture(BOOL bSave)
   0xfffa17(0x40 is the default=0x100)
   Many thanks to Steve Bak for that one!
 */
-void MFP_Exception(int Interrupt)
+static void MFP_Exception(int Interrupt)
 {
   unsigned int Vec;
 
@@ -188,25 +195,18 @@ void MFP_Exception(int Interrupt)
 /*
   Test interrupt request to see if can cause exception,return TRUE if pass vector
 */
-BOOL InterruptRequest(int Exception,unsigned char Bit,unsigned long EnableAddr,unsigned char *pPendingReg,unsigned char MaskRegister,unsigned char PriorityMaskLow,unsigned char PriorityMaskHigh,unsigned char *pInServiceReg)
+static BOOL MFP_InterruptRequest(int Exception, unsigned char Bit, unsigned char *pPendingReg, unsigned char MaskRegister,
+                                 unsigned char PriorityMaskLow, unsigned char PriorityMaskHigh, unsigned char *pInServiceReg)
 {
   /* Are any higher priority interupts in service? */
   if ( ((MFP_ISRA&PriorityMaskLow)==0) && ((MFP_ISRB&PriorityMaskHigh)==0) )
   {
-    MakeSR();
     /* Is masked? */
     if (MaskRegister&Bit)
     {
-#if 0
-      if (7>FIND_IPL)
-      {
-        *pPendingReg &= ~Bit;        /* Clear pending bit */
-        MFP_UpdateFlags();
-      }
-#endif
-
+      MakeSR();
       /* CPU allows interrupt of an MFP level? */
-      if (6>FIND_IPL)
+      if (6 > FIND_IPL)
       {
         *pPendingReg &= ~Bit;           /* Clear pending bit */
         MFP_UpdateFlags();
@@ -230,76 +230,61 @@ BOOL InterruptRequest(int Exception,unsigned char Bit,unsigned long EnableAddr,u
 
 /*-----------------------------------------------------------------------*/
 /*
-  MFP circuit has pending interrupt, request
-*/
-void MFP_RequestInterrupt_TimerA(void)
-{
-  InterruptRequest(MFP_EXCEPT_TIMERA,MFP_TIMER_A_BIT,MFP_IERA,&MFP_IPRA,MFP_IMRA,0xe0,0x00&IGNORE_FDC_PRIORITY,&MFP_ISRA);
-}
-void MFP_RequestInterrupt_TimerB(void)
-{
-  InterruptRequest(MFP_EXCEPT_TIMERB,MFP_TIMER_B_BIT,MFP_IERA,&MFP_IPRA,MFP_IMRA,0xff,0x00&IGNORE_FDC_PRIORITY,&MFP_ISRA);
-}
-void MFP_RequestInterrupt_TimerC(void)
-{
-  InterruptRequest(MFP_EXCEPT_TIMERC,MFP_TIMER_C_BIT,MFP_IERB,&MFP_IPRB,MFP_IMRB,0xff,0xe0&IGNORE_FDC_PRIORITY,&MFP_ISRB);
-}
-void MFP_RequestInterrupt_TimerD(void)
-{
-  InterruptRequest(MFP_EXCEPT_TIMERD,MFP_TIMER_D_BIT,MFP_IERB,&MFP_IPRB,MFP_IMRB,0xff,0xf0&IGNORE_FDC_PRIORITY,&MFP_ISRB);
-}
-void MFP_RequestInterrupt_Keyboard(void)
-{
-  InterruptRequest(MFP_EXCEPT_KEYBOARD,MFP_KEYBOARD_BIT,MFP_IERB,&MFP_IPRB,MFP_IMRB,0xff,0xc0&IGNORE_FDC_PRIORITY,&MFP_ISRB);
-}
-void MFP_RequestInterrupt_Floppy(void)
-{
-  InterruptRequest(MFP_EXCEPT_GPIP5,MFP_FDCHDC_BIT,MFP_IERB,&MFP_IPRB,MFP_IMRB,0xff,0x80&IGNORE_FDC_PRIORITY,&MFP_ISRB);
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Check 'pending' registers to see if any MFP interrupts need servicing
+  Check 'pending' registers to see if any MFP interrupts need servicing.
+  Request interrupt if necessary.
 */
 void MFP_CheckPendingInterrupts(void)
 {
- unsigned short i;
-
- i = (MFP_IPRA<<8) | MFP_IPRB;
-
- if( (i&0x21f0)==0 )
-  {    /* Should never get here, but if do just clear flag (see 'MFP_UpdateFlags') */
-   PendingInterruptFlag &= CLEAR_PENDING_INTERRUPT_FLAG_MFP;
-   return;
+  if ((MFP_IPRA & 0x35) == 0 && (MFP_IPRB & 0xf0) == 0)
+  { 
+    /* Should never get here, but if do just clear flag (see 'MFP_UpdateFlags') */
+    unset_special(SPCFLAG_MFP);
+    return;
   }
 
- if( i&0x2000 )  MFP_RequestInterrupt_TimerA();    /* Check Timer A (bit 5) */
- if( i&0x0100 )  MFP_RequestInterrupt_TimerB();    /* Check Timer B (bit 0) */
- if( i&0x0080 )  MFP_RequestInterrupt_Floppy();    /* Check FDC (bit 7) */
- if( i&0x0040 )  MFP_RequestInterrupt_Keyboard();  /* Check Keyboard (bit 6) */
- if( i&0x0020 )  MFP_RequestInterrupt_TimerC();    /* Check Timer C (bit 5) */
- if( i&0x0010 )  MFP_RequestInterrupt_TimerD();    /* Check Timer D (bit 4) */
+  if (MFP_IPRA & MFP_TIMER_A_BIT)       /* Check Timer A (bit 5) */
+    MFP_InterruptRequest(MFP_EXCEPT_TIMERA, MFP_TIMER_A_BIT, &MFP_IPRA, MFP_IMRA, 0xe0, 0x00, &MFP_ISRA);
+
+  if (MFP_IPRA & MFP_RCVBUFFULL_BIT)    /* Check Receive buffer full (bit 4) */
+    MFP_InterruptRequest(MFP_EXCEPT_RECBUFFULL, MFP_RCVBUFFULL_BIT, &MFP_IPRA, MFP_IMRA, 0xf0, 0x00, &MFP_ISRA);
+
+  if (MFP_IPRA & MFP_TRNBUFEMPTY_BIT)   /* Check transmit buffer empty (bit 2) */
+    MFP_InterruptRequest(MFP_EXCEPT_TRANSBUFFEMPTY, MFP_TRNBUFEMPTY_BIT, &MFP_IPRA, MFP_IMRA, 0xfb, 0x00, &MFP_ISRA);
+
+  if (MFP_IPRA & MFP_TIMER_B_BIT)       /* Check Timer B (bit 0) */
+    MFP_InterruptRequest(MFP_EXCEPT_TIMERB, MFP_TIMER_B_BIT, &MFP_IPRA, MFP_IMRA, 0xff, 0x00, &MFP_ISRA);
+
+  if (MFP_IPRB & MFP_FDCHDC_BIT)        /* Check FDC (bit 7) */
+    MFP_InterruptRequest(MFP_EXCEPT_GPIP5, MFP_FDCHDC_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0x80, &MFP_ISRB);
+
+  if (MFP_IPRB & MFP_ACIA_BIT)          /* Check ACIA (Keyboard or MIDI) (bit 6) */
+    MFP_InterruptRequest(MFP_EXCEPT_ACIA, MFP_ACIA_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xc0, &MFP_ISRB);
+
+  if (MFP_IPRB & MFP_TIMER_C_BIT)       /* Check Timer C (bit 5) */
+    MFP_InterruptRequest(MFP_EXCEPT_TIMERC, MFP_TIMER_C_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xe0, &MFP_ISRB);
+
+  if (MFP_IPRB & MFP_TIMER_D_BIT)       /* Check Timer D (bit 4) */
+    MFP_InterruptRequest(MFP_EXCEPT_TIMERD, MFP_TIMER_D_BIT, &MFP_IPRB, MFP_IMRB, 0xff, 0xf0, &MFP_ISRB);
 }
 
 
 /*-----------------------------------------------------------------------*/
 /*
   This is called whenever the MFP_IPRA or MFP_IPRB registers are modified.
-  We set the 'PendingInterruptFlag' accordingly (to say if an MFP interrupt
+  We set the special flag SPCFLAG_MFP accordingly (to say if an MFP interrupt
   is to be checked) so we only have one compare during the decode
   instruction loop.
 */
 void MFP_UpdateFlags(void)
 {
- if( MFP_IPRA|MFP_IPRB )
-   {
-    PendingInterruptFlag |= PENDING_INTERRUPT_FLAG_MFP;
-   }
+  if( MFP_IPRA|MFP_IPRB )
+  {
+    set_special(SPCFLAG_MFP);
+  }
   else
-   {
-    PendingInterruptFlag &= CLEAR_PENDING_INTERRUPT_FLAG_MFP;
-   }
+  {
+    unset_special(SPCFLAG_MFP);
+  }
 }
 
 
@@ -356,7 +341,7 @@ void MFP_TimerB_EventCount_Interrupt(void)
 /*
   Start Timer A or B - EventCount mode is done in HBL handler to time correctly
 */
-int MFP_StartTimer_AB(unsigned char TimerControl, unsigned int TimerData, int Handler, BOOL bFirstTimer)
+static int MFP_StartTimer_AB(unsigned char TimerControl, unsigned int TimerData, int Handler, BOOL bFirstTimer)
 {
   int TimerClockCycles = 0;
 
@@ -390,7 +375,7 @@ int MFP_StartTimer_AB(unsigned char TimerControl, unsigned int TimerData, int Ha
 /*
   Start Timer C or D
 */
-int MFP_StartTimer_CD(unsigned char TimerControl, unsigned int TimerData, int Handler, BOOL bFirstTimer)
+static int MFP_StartTimer_CD(unsigned char TimerControl, unsigned int TimerData, int Handler, BOOL bFirstTimer)
 {
   int TimerClockCycles = 0;
 
@@ -424,7 +409,7 @@ int MFP_StartTimer_CD(unsigned char TimerControl, unsigned int TimerData, int Ha
 /*
   Read Timer A or B - If in EventCount MainCounter already has correct value
 */
-unsigned char MFP_ReadTimer_AB(unsigned char TimerControl, unsigned char MainCounter, int TimerCycles, int Handler)
+static unsigned char MFP_ReadTimer_AB(unsigned char TimerControl, unsigned char MainCounter, int TimerCycles, int Handler)
 {
   int TimerCyclesPassed;
 
@@ -443,7 +428,7 @@ unsigned char MFP_ReadTimer_AB(unsigned char TimerControl, unsigned char MainCou
 /*
   Read Timer C or D
 */
-unsigned char MFP_ReadTimerCD(unsigned char TimerControl,unsigned char TimerData,  unsigned char MainCounter, int TimerCycles, int Handler)
+static unsigned char MFP_ReadTimerCD(unsigned char TimerControl,unsigned char TimerData,  unsigned char MainCounter, int TimerCycles, int Handler)
 {
   int TimerCyclesPassed;
 
