@@ -19,7 +19,7 @@
   only convert the screen every 50 times a second - inbetween frames are not
   processed.
 */
-static char rcsid[] = "Hatari $Id: screen.c,v 1.27 2003/06/17 18:03:22 thothy Exp $";
+char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.35 2004/12/05 23:30:17 thothy Exp $";
 
 #include <SDL.h>
 
@@ -40,24 +40,21 @@ static char rcsid[] = "Hatari $Id: screen.c,v 1.27 2003/06/17 18:03:22 thothy Ex
 #include "video.h"
 
 /* SDL Video attributes for fullscreen */
-const unsigned int sdlvmode = SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN|SDL_HWPALETTE;
+static const unsigned int sdlvmode = SDL_HWSURFACE|/*SDL_DOUBLEBUF|*/SDL_FULLSCREEN|SDL_HWPALETTE;
+/* SDL_DOUBLEBUF is a good idea, but the GUI doesn't work with double buffered
+ * screens yet, so double buffering is currently disabled. */
 
-SCREENDRAW ScreenDrawWindow[4];                   /* Set up with details of drawing functions for ST_xxx_RES */
 SCREENDRAW ScreenDrawFullScreen[4];               /* And for full-screen */
-SCREENDRAW ScreenDrawVDIWindow[4];
 SCREENDRAW ScreenDrawVDIFullScreen[4];            /* And for full-screen */
 FRAMEBUFFER FrameBuffers[NUM_FRAMEBUFFERS];       /* Store frame buffer details to tell how to update */
 FRAMEBUFFER *pFrameBuffer;                        /* Pointer into current 'FrameBuffer' */
-unsigned char *pScreenBitmap=NULL;                /* Screen pixels in PC RGB format, allocated with 'CreateDIBSection' */
 unsigned char *pSTScreen,*pSTScreenCopy;          /* Keep track of current and previous ST screen data */
 unsigned char *pPCScreenDest;                     /* Destination PC buffer */
 int STScreenStartHorizLine,STScreenEndHorizLine;  /* Start/End lines to be converted */
 int PCScreenBytesPerLine, STScreenWidthBytes, STScreenLeftSkipBytes;
 BOOL bInFullScreen=FALSE;                         /* TRUE if in full screen */
-BOOL bFullScreenHold = FALSE;                     /* TRUE if hold display while full screen */
 BOOL bScreenContentsChanged;                      /* TRUE if buffer changed and requires blitting */
 int STRes=ST_LOW_RES, PrevSTRes=ST_LOW_RES;       /* Current and previous ST resolutions */
-int nDroppedFrames=0;                             /* Number of dropped frames during emulation run */
 
 int STScreenLineOffset[NUM_VISIBLE_LINES];        /* Offsets for ST screen lines eg, 0,160,320... */
 unsigned long STRGBPalette[16];                   /* Palette buffer used in assembler conversion routines */
@@ -76,25 +73,30 @@ static void Screen_SetWindowRes()
 {
   int Width, Height, BitCount;
 
-  if (bUseVDIRes) {
+  if (bUseVDIRes)
+  {
     Width = VDIWidth;
     Height = VDIHeight;
-  } else
+  }
+  else
+  {
     switch(STRes)
-      {
+    {
       case ST_LOW_RES:
-	if (ConfigureParams.Screen.ChosenDisplayMode == 0 ||
-	    ConfigureParams.Screen.ChosenDisplayMode == 3) {
-	  Width = 320;
-	  Height = 200;
-	  break;
-	}
-	// else use 640x400
+        if (ConfigureParams.Screen.ChosenDisplayMode == DISPLAYMODE_LOWCOL_LOWRES
+            || ConfigureParams.Screen.ChosenDisplayMode == DISPLAYMODE_HICOL_LOWRES)
+        {
+          Width = 320;
+          Height = 200;
+          break;
+        }
+        /* else use 640x400 */
       default:
-	Width = 640;
-	Height = 400;
-	break;
-      }
+        Width = 640;
+        Height = 400;
+        break;
+    }
+  }
 
   /* Adjust width/height for overscan borders, if mono or VDI we have no overscan */
 
@@ -113,8 +115,9 @@ static void Screen_SetWindowRes()
   }
 
   /* Bits per pixel */
-  if(ConfigureParams.Screen.ChosenDisplayMode <= DISPLAYMODE_16COL_FULL
-     || STRes == ST_HIGH_RES || bUseVDIRes)
+  if (ConfigureParams.Screen.ChosenDisplayMode == DISPLAYMODE_LOWCOL_LOWRES
+      || ConfigureParams.Screen.ChosenDisplayMode == DISPLAYMODE_LOWCOL_HIGHRES
+      || STRes == ST_HIGH_RES || bUseVDIRes)
   {
     BitCount = 8;
   }
@@ -130,7 +133,6 @@ static void Screen_SetWindowRes()
     SDL_Quit();
     exit(-2);
   }
-  pScreenBitmap=sdlscrn->pixels;
 
   if(BitCount==8)
   {
@@ -148,6 +150,7 @@ static void Screen_SetWindowRes()
   Screen_SetDrawModes();        /* Set draw modes(store which modes to use!) */
 }
 
+
 /*-----------------------------------------------------------------------*/
 /*
   Init Screen bitmap and buffers/tables needed for ST to PC screen conversion
@@ -333,7 +336,6 @@ void Screen_EnterFullScreen(void)
     else
     {
       sdlscrn = newsdlscrn;
-      pScreenBitmap = newsdlscrn->pixels;
       bInFullScreen = TRUE;
 
       Screen_SetFullUpdate();           /* Cause full update of screen */
@@ -393,41 +395,11 @@ void Screen_SetDrawModes(void)
   SCREENDRAW_DISPLAYOPTIONS *pScreenDisplay;
 
   /* Clear out */
-  Memory_Clear(ScreenDrawWindow,sizeof(SCREENDRAW)*4);
   Memory_Clear(ScreenDrawFullScreen,sizeof(SCREENDRAW)*4);
-  Memory_Clear(ScreenDrawVDIWindow,sizeof(SCREENDRAW)*4);
   Memory_Clear(ScreenDrawVDIFullScreen,sizeof(SCREENDRAW)*4);
 
-  /* First, store Window details(set for 16-bit Windows desktop) */
-  ScreenDrawWindow[ST_LOW_RES].pDrawFunction = ConvertLowRes_320x16Bit;
-  ScreenDrawWindow[ST_LOW_RES].Width = 320;     ScreenDrawWindow[ST_LOW_RES].Height = 200;     ScreenDrawWindow[ST_LOW_RES].BitDepth = 16;
-
-  ScreenDrawWindow[ST_MEDIUM_RES].pDrawFunction = ConvertMediumRes_640x16Bit;
-  ScreenDrawWindow[ST_MEDIUM_RES].Width = 640;  ScreenDrawWindow[ST_MEDIUM_RES].Height = 200;  ScreenDrawWindow[ST_MEDIUM_RES].BitDepth = 16;
-
-  ScreenDrawWindow[ST_HIGH_RES].pDrawFunction = ConvertHighRes_640x8Bit;
-  ScreenDrawWindow[ST_HIGH_RES].Width = 640;    ScreenDrawWindow[ST_HIGH_RES].Height = 400;    ScreenDrawWindow[ST_HIGH_RES].BitDepth = 8;
-
-  /* (NOTE this is irrelevant as is directed to low/medium when starts) */
-  ScreenDrawWindow[ST_LOWMEDIUM_MIX_RES].pDrawFunction = ConvertLowRes_640x16Bit;
-  ScreenDrawWindow[ST_LOWMEDIUM_MIX_RES].Width = 640;  ScreenDrawWindow[ST_LOWMEDIUM_MIX_RES].Height = 200;  ScreenDrawWindow[ST_LOWMEDIUM_MIX_RES].BitDepth = 16;
-
-  /* And for VDI screens (set for 8-bit) */
-  ScreenDrawVDIWindow[ST_LOW_RES].pDrawFunction = ConvertVDIRes_16Colour;
-  ScreenDrawVDIWindow[ST_LOW_RES].Width = VDIWidth;  ScreenDrawVDIWindow[ST_LOW_RES].Height = VDIHeight;  ScreenDrawVDIWindow[ST_LOW_RES].BitDepth = 8;
-  ScreenDrawVDIWindow[ST_MEDIUM_RES].pDrawFunction = ConvertVDIRes_4Colour;
-  ScreenDrawVDIWindow[ST_MEDIUM_RES].Width = VDIWidth;  ScreenDrawVDIWindow[ST_MEDIUM_RES].Height = VDIHeight;  ScreenDrawVDIWindow[ST_MEDIUM_RES].BitDepth = 8;
-  ScreenDrawVDIWindow[ST_HIGH_RES].pDrawFunction = ConvertVDIRes_2Colour;
-  ScreenDrawVDIWindow[ST_HIGH_RES].Width = VDIWidth;  ScreenDrawVDIWindow[ST_HIGH_RES].Height = VDIHeight;  ScreenDrawVDIWindow[ST_HIGH_RES].BitDepth = 8;
-
-  /* And full-screen, select from Overscan/Non-Overscan */
-  if (ConfigureParams.Screen.bAllowOverscan) {
-    pScreenDisplay = &ScreenDisplayOptions[ConfigureParams.Screen.ChosenDisplayMode];
-  } else {
-    pScreenDisplay = &ScreenDisplayOptions_NoOverscan[ConfigureParams.Screen.ChosenDisplayMode];
-  }
-
   /* Assign full-screen draw modes from chosen option under dialog */
+  pScreenDisplay = &ScreenDisplayOptions[ConfigureParams.Screen.ChosenDisplayMode];
   ScreenDrawFullScreen[ST_LOW_RES] = *pScreenDisplay->pLowRes;
   ScreenDrawFullScreen[ST_MEDIUM_RES] = *pScreenDisplay->pMediumRes;
   ScreenDrawFullScreen[ST_HIGH_RES] = *pScreenDisplay->pHighRes;
@@ -505,7 +477,7 @@ void Screen_DidResolutionChange(void)
   Compare current resolution on line with previous, and set 'UpdateLine' accordingly
   Also check if swap between low/medium resolution and return in 'bLowMedMix'
 */
-void Screen_CompareResolution(int y, int *pUpdateLine, BOOL *pbLowMedMix)
+static void Screen_CompareResolution(int y, int *pUpdateLine, BOOL *pbLowMedMix)
 {
   int Resolution;
 
@@ -529,7 +501,7 @@ void Screen_CompareResolution(int y, int *pUpdateLine, BOOL *pbLowMedMix)
 /*
   Check to see if palette changes cause screen update and keep 'HBLPalette[]' up-to-date
 */
-void Screen_ComparePalette(int y, int *pUpdateLine)
+static void Screen_ComparePalette(int y, int *pUpdateLine)
 {
   BOOL bPaletteChanged = FALSE;
   int i;
@@ -564,7 +536,7 @@ void Screen_ComparePalette(int y, int *pUpdateLine)
   (It is very important for these routines to check for colour changes with the previous
   screen so only the very minimum parts are updated)
 */
-int Screen_ComparePaletteMask(void)
+static int Screen_ComparePaletteMask(void)
 {
   BOOL bLowMedMix=FALSE;
   int LineUpdate = 0;
@@ -647,7 +619,7 @@ int Screen_ComparePaletteMask(void)
 /*
   Create 8-Bit palette for display if needed
 */
-void Screen_CreatePalette(void)
+static void Screen_CreatePalette(void)
 {
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
   static const int endiantable[16] = {0,2,1,3,8,10,9,11,4,6,5,7,12,14,13,15};
@@ -735,7 +707,7 @@ void Screen_Handle8BitPalettes(void)
   Update Palette Mask to show 'full-update' required. This is usually done after a resolution change
   or when going between a Window and full-screen display
 */
-void Screen_SetFullUpdateMask(void)
+static void Screen_SetFullUpdateMask(void)
 {
   int y;
 
@@ -748,14 +720,14 @@ void Screen_SetFullUpdateMask(void)
 /*
   Set details for ST screen conversion(Window version)
 */
-void Screen_SetWindowConvertDetails(void)
+static void Screen_SetWindowConvertDetails(void)
 {
   /* Reset Double Y, used for Window medium res' and in full screen */
   bScrDoubleY = FALSE;
 
   pSTScreen = pFrameBuffer->pSTScreen;          /* Source in ST memory */
   pSTScreenCopy = pFrameBuffer->pSTScreenCopy;  /* Previous ST screen */
-  pPCScreenDest = pScreenBitmap;                /* Destination PC screen */
+  pPCScreenDest = sdlscrn->pixels;              /* Destination PC screen */
 
   STScreenStartHorizLine = 0;                   /* Full height */
 
@@ -787,7 +759,7 @@ void Screen_SetWindowConvertDetails(void)
     pHBLPalettes = pFrameBuffer->HBLPalettes;
   }
 
-  if (!ConfigureParams.Screen.bInterlacedScreen)
+  if (!ConfigureParams.Screen.bInterleavedScreen)
   {
     bScrDoubleY = TRUE;
   }
@@ -797,7 +769,7 @@ void Screen_SetWindowConvertDetails(void)
 /*
   Set details for ST screen conversion (Full-Screen version)
 */
-void Screen_SetFullScreenConvertDetails(void)
+static void Screen_SetFullScreenConvertDetails(void)
 {
   SCREENDRAW *pScreenDraw;
 
@@ -828,8 +800,8 @@ void Screen_SetFullScreenConvertDetails(void)
   pPCScreenDest += pScreenDraw->Overscan[OverscanMode].PCStartXOffset;
   pHBLPalettes = pFrameBuffer->HBLPalettes;
 
-  /* Is non-interlaced? May need to double up on Y */
-  if (!ConfigureParams.Screen.bInterlacedScreen)
+  /* Is non-interleaved? May need to double up on Y */
+  if (!ConfigureParams.Screen.bInterleavedScreen)
   {
     bScrDoubleY = TRUE;
   }
@@ -840,7 +812,7 @@ void Screen_SetFullScreenConvertDetails(void)
 /*
   Lock full-screen for drawing
 */
-BOOL Screen_Lock(void)
+static BOOL Screen_Lock(void)
 {
   if(SDL_MUSTLOCK(sdlscrn))
   {
@@ -858,7 +830,7 @@ BOOL Screen_Lock(void)
 /*
   UnLock full-screen
 */
-void Screen_UnLock(void)
+static void Screen_UnLock(void)
 {
   if( SDL_MUSTLOCK(sdlscrn) )
     SDL_UnlockSurface(sdlscrn);
@@ -939,10 +911,10 @@ void Screen_Blit(BOOL bSwapScreen)
 /*
   Swap ST Buffers, used for full-screen where have double-buffering
 */
-void Screen_SwapSTBuffers(void)
+static void Screen_SwapSTBuffers(void)
 {
 #if NUM_FRAMEBUFFERS > 1
-  if (bInFullScreen)
+  if (sdlscrn->flags & SDL_DOUBLEBUF)
   {
     if (pFrameBuffer==&FrameBuffers[0])
       pFrameBuffer = &FrameBuffers[1];
@@ -960,7 +932,6 @@ void Screen_SwapSTBuffers(void)
 void Screen_DrawFrame(BOOL bForceFlip)
 {
   void *pDrawFunction;
-  static void *last;
 
   /* Scan palette/resolution masks for each line and build up palette/difference tables */
   STRes = Screen_ComparePaletteMask();
@@ -979,13 +950,12 @@ void Screen_DrawFrame(BOOL bForceFlip)
     if (ConfigureParams.Screen.bAllowOverscan)
     {
       Screen_SetWindowConvertDetails();
-      pPCScreenDest = (unsigned char *)sdlscrn->pixels;  /* Destination PC screen */
     }
     else
     {
       Screen_SetFullScreenConvertDetails();
     }
-    /* Clear screen on full update to clear out borders and also interlaced lines */
+    /* Clear screen on full update to clear out borders and also interleaved lines */
     if (pFrameBuffer->bFullUpdate && !bUseVDIRes)
       Screen_ClearScreen();
     /* Call drawing for full-screen */
@@ -1026,46 +996,15 @@ void Screen_DrawFrame(BOOL bForceFlip)
   }
 }
 
+
 /*-----------------------------------------------------------------------*/
 /*
   Draw ST screen to window/full-screen
 */
 void Screen_Draw(void)
 {
-  /* Are we holding screen? Ie let user choose options while in full-screen mode using GDI */
-  if (bInFullScreen && bFullScreenHold)
-  {
-    /* Just update status bar */
-    /*StatusBar_UpdateIcons();*/ /* No statusbar in Hatari */
-    return;
-  }
-
   if (!bQuitProgram)
   {
-#if 0
-    /* Wait for next display(at 50fps), is ignored if running max speed */
-    if ( !(ConfigureParams.Screen.bSyncToRetrace && bInFullScreen) )
-    {
-      /* If in Max speed mode, just get on with it, or else wait for VBL timer */
-      if (ConfigureParams.Configure.nMinMaxSpeed!=MINMAXSPEED_MAX)
-      {
-        /* Has VBL already occured? Means we can't keep 50fps to warn user */
-        if (Main_AlreadyWaitingVBLEvent())
-        {
-          /* Increase counter for number of consecutive dropped frames */
-          nDroppedFrames++;
-          /* If emulation has gone slow for 1/2 second or more, inform user */
-          /*if (nDroppedFrames>=25)
-            StatusBar_SetIcon(STATUS_ICON_FRAMERATE,ICONSTATE_UPDATE);*/ /* No statusbar in Hatari yet */
-        }
-        else
-          nDroppedFrames = 0;
-        /* Wait for VBL event */
-        Main_WaitVBLEvent();
-      }
-    }
-#endif
-
     if(VideoBase)
     {
       /* And draw(if screen contents changed) */
@@ -1078,5 +1017,4 @@ void Screen_Draw(void)
     /* Check printer status */
     Printer_CheckIdleStatus();
   }
-
 }
