@@ -17,7 +17,7 @@
   improves speed. We then look these bbp values up as an RGB/Index value to copy
   to the screen.
 */
-static char rcsid[] = "Hatari $Id: screenConvert.c,v 1.14 2003/02/02 22:41:34 thothy Exp $";
+char ScreenConvert_rcsid[] = "Hatari $Id: screenConvert.c,v 1.15 2004/04/19 08:53:34 thothy Exp $";
 
 #include <SDL.h>
 #include <SDL_endian.h>
@@ -30,13 +30,14 @@ static char rcsid[] = "Hatari $Id: screenConvert.c,v 1.14 2003/02/02 22:41:34 th
 #include "video.h"
 
 
-int ScrX,ScrY;                   /* Locals */
 int ScrUpdateFlag;               /* Bit mask of how to update screen */
 BOOL bScrDoubleY;                /* TRUE if double on Y */
-Uint32 PixelWorkspace[4];        /* Workspace to store pixels to so can print in right order for Spec512 */
+
+static int ScrX,ScrY;                   /* Locals */
+static Uint32 PixelWorkspace[4];        /* Workspace to store pixels to so can print in right order for Spec512 */
 
 /* Remap tables to convert from plane format to byte-per-pixel (Upper is for 4-Planes so if shifted by 2) */
-Uint32 Remap_2_Planes[256] = {
+static Uint32 Remap_2_Planes[256] = {
   0x00000000,  0x01000000,  0x00010000,  0x01010000,  0x00000100,  0x01000100,  0x00010100,  0x01010100,
   0x00000001,  0x01000001,  0x00010001,  0x01010001,  0x00000101,  0x01000101,  0x00010101,  0x01010101,
   0x02000000,  0x03000000,  0x02010000,  0x03010000,  0x02000100,  0x03000100,  0x02010100,  0x03010100,
@@ -71,7 +72,7 @@ Uint32 Remap_2_Planes[256] = {
   0x02020203,  0x03020203,  0x02030203,  0x03030203,  0x02020303,  0x03020303,  0x02030303,  0x03030303,
 };
 
-Uint32 Remap_2_Planes_Upper[256] = {
+static Uint32 Remap_2_Planes_Upper[256] = {
   0x00000000,  0x04000000,  0x00040000,  0x04040000,  0x00000400,  0x04000400,  0x00040400,  0x04040400,
   0x00000004,  0x04000004,  0x00040004,  0x04040004,  0x00000404,  0x04000404,  0x00040404,  0x04040404,
   0x08000000,  0x0C000000,  0x08040000,  0x0C040000,  0x08000400,  0x0C000400,  0x08040400,  0x0C040400,
@@ -106,7 +107,7 @@ Uint32 Remap_2_Planes_Upper[256] = {
   0x0808080C,  0x0C08080C,  0x080C080C,  0x0C0C080C,  0x08080C0C,  0x0C080C0C,  0x080C0C0C,  0x0C0C0C0C,
 };
 
-Uint32 Remap_1_Plane[16] = {
+static Uint32 Remap_1_Plane[16] = {
   0x00000000+BASECOLOUR_LONG,  0x01000000+BASECOLOUR_LONG,  0x00010000+BASECOLOUR_LONG,  0x01010000+BASECOLOUR_LONG,  0x00000100+BASECOLOUR_LONG,  0x01000100+BASECOLOUR_LONG,  0x00010100+BASECOLOUR_LONG,  0x01010100+BASECOLOUR_LONG,
   0x00000001+BASECOLOUR_LONG,  0x01000001+BASECOLOUR_LONG,  0x00010001+BASECOLOUR_LONG,  0x01010001+BASECOLOUR_LONG,  0x00000101+BASECOLOUR_LONG,  0x01000101+BASECOLOUR_LONG,  0x00010101+BASECOLOUR_LONG,  0x01010101+BASECOLOUR_LONG,
 };
@@ -117,7 +118,7 @@ Uint32 Remap_1_Plane[16] = {
 
   Return 'ScrUpdateFlag', 0x80000000=Full update, 0x40000000=Update as palette changed
 */
-int AdjustLinePaletteRemap(void)
+static int AdjustLinePaletteRemap(void)
 {
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
   static const int endiantable[16] = {0,2,1,3,8,10,9,11,4,6,5,7,12,14,13,15};
@@ -148,7 +149,7 @@ int AdjustLinePaletteRemap(void)
 /*
   Run updates to palette(STRGBPalette[]) until get to screen line we are to convert from
 */
-void Convert_StartFrame(void)
+static void Convert_StartFrame(void)
 {
  int ecx;
  ecx=STScreenStartHorizLine;            /* Get #lines before conversion starts */
@@ -570,11 +571,6 @@ void Convert_StartFrame(void)
   ebx = STRGBPalette[ecx]; \
   esi[offset] = ebx; \
 }
-/*
-	__asm	and		ecx,0x000000ff \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ecx*4] \
-	__asm	mov		offset[esi],ebx
-*/
 
 /* Plot Spectrum512 Resolution (640xH) 16-Bit pixels */
 #define PLOT_SPEC512_MID_640_16BIT(offset)	\
@@ -619,23 +615,6 @@ void Convert_StartFrame(void)
   ebx = STRGBPalette[ebx]; \
   esi[offset+2] = ebx; \
 }
-/*
-	__asm	mov		ebx,ecx \
-	__asm	and		ebx,0x000000ff \
-	__asm	shr		ecx,8 \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ebx*4] \
-	__asm	mov		offset[esi],ebx \
-	__asm	mov		ebx,ecx \
-	__asm	and		ebx,0x000000ff \
-	__asm	shr		ecx,8 \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ebx*4] \
-	__asm	mov		offset[esi+4],ebx \
-	__asm	mov		ebx,ecx \
-	__asm	and		ebx,0x000000ff \
-	__asm	shr		ecx,8 \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ebx*4] \
-	__asm	mov		offset[esi+8],ebx
-*/
 
 /* Plot Spectrum512 Resolution (640xH) 16-Bit pixels (Double on Y) */
 #define PLOT_SPEC512_LEFT_LOW_640_16BIT_DOUBLE_Y(offset)	\
@@ -645,12 +624,6 @@ void Convert_StartFrame(void)
   esi[offset] = ebx; \
   esi[offset+PCScreenBytesPerLine/4] = ebx; \
 }
-/*
-	__asm	and		ecx,0x000000ff \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ecx*4] \
-	__asm	mov		offset[esi],ebx \
-	__asm	mov		offset[esi+ebp],ebx
-*/
 
 /* Plot Spectrum512 Resolution (640xH) 16-Bit pixels (Double on Y) */
 #define PLOT_SPEC512_MID_640_16BIT_DOUBLE_Y(offset)	\
@@ -702,26 +675,6 @@ void Convert_StartFrame(void)
   esi[offset+2] = ebx; \
   esi[offset+2+PCScreenBytesPerLine/4] = ebx; \
 }
-/*
-	__asm	mov		ebx,ecx \
-	__asm	and		ebx,0x000000ff \
-	__asm	shr		ecx,8 \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ebx*4] \
-	__asm	mov		offset[esi],ebx \
-	__asm	mov		offset[esi+ebp],ebx \
-	__asm	mov		ebx,ecx \
-	__asm	and		ebx,0x000000ff \
-	__asm	shr		ecx,8 \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ebx*4] \
-	__asm	mov		offset[esi+4],ebx \
-	__asm	mov		offset[esi+4+ebp],ebx \
-	__asm	mov		ebx,ecx \
-	__asm	and		ebx,0x000000ff \
-	__asm	shr		ecx,8 \
-	__asm	mov		ebx,DWORD PTR STRGBPalette[ebx*4] \
-	__asm	mov		offset[esi+8],ebx \
-	__asm	mov		offset[esi+8+ebp],ebx
-*/
 
 
 
