@@ -10,16 +10,16 @@
   * This file is distributed under the GNU Public License, version 2 or at
   * your option any later version. Read the file gpl.txt for details.
   */
-static char rcsid[] = "Hatari $Id: newcpu.c,v 1.29 2003/10/25 12:26:39 thothy Exp $";
+char NewCpu_rcsid[] = "Hatari $Id: newcpu.c,v 1.36 2004/12/08 10:27:53 thothy Exp $";
 
 #include "sysdeps.h"
 #include "hatari-glue.h"
 #include "maccess.h"
 #include "memory.h"
 #include "newcpu.h"
-#include "events.h"
 #include "../includes/main.h"
 #include "../includes/m68000.h"
+#include "../includes/mfp.h"
 #include "../includes/tos.h"
 #include "../includes/vdi.h"
 #include "../includes/cart.h"
@@ -153,7 +153,6 @@ void build_cpufunctbl(void)
 
     /* Hatari's illegal opcodes: */
     cpufunctbl[GEMDOS_OPCODE] = OpCode_GemDos;
-    cpufunctbl[RUNOLDGEMDOS_OPCODE] = OpCode_OldGemDos;
     cpufunctbl[SYSINIT_OPCODE] = OpCode_SysInit;
     cpufunctbl[VDI_OPCODE] = OpCode_VDI;
 }
@@ -233,8 +232,10 @@ void init_m68k (void)
 
 
 struct regstruct regs, lastint_regs;
+/* not used ATM:
 static struct regstruct regs_backup[16];
 static int backup_pointer = 0;
+*/
 static long int m68kpc_offset;
 int lastint_no;
 
@@ -414,9 +415,11 @@ uae_s32 ShowEA (FILE *f, int reg, amodes mode, wordsizes size, char *buf)
     return offset;
 }
 
+
 /* The plan is that this will take over the job of exception 3 handling -
  * the CPU emulation functions will just do a longjmp to m68k_go whenever
  * they hit an odd address. */
+#if 0
 static int verify_ea (int reg, amodes mode, wordsizes size, uae_u32 *val)
 {
     uae_u16 dp;
@@ -537,6 +540,8 @@ static int verify_ea (int reg, amodes mode, wordsizes size, uae_u32 *val)
     last_fault_for_exception_3 = addr;
     return 0;
 }
+#endif
+
 
 uae_u32 get_disp_ea_020 (uae_u32 base, uae_u32 dp)
 {
@@ -707,6 +712,7 @@ void Exception(int nr, uaecptr oldpc)
 
     MakeSR();
 
+    /* Change to supervisor mode if necessary */
     if (!regs.s) {
 	regs.usp = m68k_areg(regs, 7);
 	if (cpu_level >= 2)
@@ -715,6 +721,8 @@ void Exception(int nr, uaecptr oldpc)
 	    m68k_areg(regs, 7) = regs.isp;
 	regs.s = 1;
     }
+
+    /* Build additional exception stack frame for 68010 and higher */
     if (cpu_level > 0) {
 	if (nr == 2 || nr == 3) {
 	    int i;
@@ -756,32 +764,50 @@ void Exception(int nr, uaecptr oldpc)
     put_word (m68k_areg(regs, 7), regs.sr);
 
     /* 68000 bus/address errors: */
-    /* Well these 8 more bytes are not here just for debuging.
-       It seems adebug expects them to be on the stack when it receives
-       a bus error... */
     if (cpu_level==0 && (nr==2 || nr==3)) {
+	uae_u16 specialstatus = 0x2001;
+	/* Special status word emulation isn't perfect yet... :-( */
+	if (regs.sr & 0x2000)
+	    specialstatus |= 0x4;
 	m68k_areg(regs, 7) -= 8;
 	if (nr == 3) {    /* Address error */
-            put_word (m68k_areg(regs, 7), 0);  /* FIXME: Add real function code value */
+	    put_word (m68k_areg(regs, 7), specialstatus);
 	    put_long (m68k_areg(regs, 7)+2, last_fault_for_exception_3);
 	    put_word (m68k_areg(regs, 7)+6, last_op_for_exception_3);
 	    put_long (m68k_areg(regs, 7)+10, last_addr_for_exception_3);
-            if( bEnableDebug ) {
-              fprintf(stderr,"Address Error at address $%x, PC=$%x\n",last_fault_for_exception_3,currpc);
-              DebugUI();
-            }
+	    if (bEnableDebug) {
+	      fprintf(stderr,"Address Error at address $%x, PC=$%x\n",last_fault_for_exception_3,currpc);
+	      DebugUI();
+	    }
 	}
-        else {    /* Bus error */
-            put_word (m68k_areg(regs, 7), 0);  /* FIXME: Add real function code value */
+	else {    /* Bus error */
+	    if (bBusErrorReadWrite)
+	      specialstatus |= 0x10;
+	    put_word (m68k_areg(regs, 7), specialstatus);
 	    put_long (m68k_areg(regs, 7)+2, BusAddressLocation);
-	    put_word (m68k_areg(regs, 7)+6, BusErrorOpcode);
-            if( bEnableDebug && BusAddressLocation!=0xff8a00) {
-              fprintf(stderr,"Bus Error at address $%x, PC=$%lx\n",BusAddressLocation,(long)currpc);
-              DebugUI();
-            }
-        }
+	    put_word (m68k_areg(regs, 7)+6, get_word(BusErrorPC));  /* Opcode */
+	    /* Check for double bus errors: */
+	    if (regs.spcflags & SPCFLAG_BUSERROR) {
+	      fprintf(stderr, "Detected double bus error at address $%x, PC=$%lx => CPU halted!\n",
+	              BusAddressLocation, (long)currpc);
+	      unset_special(SPCFLAG_BUSERROR);
+	      if (bEnableDebug)
+	        DebugUI();
+	      regs.intmask = 7;
+	      m68k_setstopped(TRUE);
+	      return;
+	    }
+	    if (bEnableDebug && BusAddressLocation!=0xff8a00) {
+	      fprintf(stderr,"Bus Error at address $%x, PC=$%lx\n",BusAddressLocation,(long)currpc);
+	      DebugUI();
+	    }
+	}
     }
 
+    /* Set PC and flags */
+    if (bEnableDebug && get_long (regs.vbr + 4*nr) == 0) {
+        write_log("Uninitialized exception handler #%i!\n", nr);
+    }
     m68k_setpc (get_long (regs.vbr + 4*nr));
     fill_prefetch_0 ();
     regs.t1 = regs.t0 = regs.m = 0;
@@ -790,30 +816,30 @@ void Exception(int nr, uaecptr oldpc)
     /* Handle exception cycles: */
     if(nr >= 24 && nr <= 31)
     {
-      ADD_CYCLES(44+4, 5, 3);                 /* Interrupt */
+      M68000_AddCycles(44+4);                 /* Interrupt */
     }
     else if(nr >= 32 && nr <= 47)
     {
-      ADD_CYCLES(34, 5, 3);                   /* Trap */
+      M68000_AddCycles(34);                   /* Trap */
     }
     else switch(nr)
     {
-      case 2: ADD_CYCLES(50, 4, 7); break;    /* Bus error */
-      case 3: ADD_CYCLES(50, 4, 7); break;    /* Address error */
-      case 4: ADD_CYCLES(34, 4, 3); break;    /* Illegal instruction */
-      case 5: ADD_CYCLES(38, 4, 3); break;    /* Div by zero */
-      case 6: ADD_CYCLES(40, 4, 3); break;    /* CHK */
-      case 7: ADD_CYCLES(34, 5, 3); break;    /* TRAPV */
-      case 8: ADD_CYCLES(34, 4, 3); break;    /* Privilege violation */
-      case 9: ADD_CYCLES(34, 4, 3); break;    /* Trace */
-      case 10: ADD_CYCLES(34, 4, 3); break;   /* Line-A - probably wrong */
-      case 11: ADD_CYCLES(34, 4, 3); break;   /* Line-F - probably wrong */
+      case 2: M68000_AddCycles(50); break;    /* Bus error */
+      case 3: M68000_AddCycles(50); break;    /* Address error */
+      case 4: M68000_AddCycles(34); break;    /* Illegal instruction */
+      case 5: M68000_AddCycles(38); break;    /* Div by zero */
+      case 6: M68000_AddCycles(40); break;    /* CHK */
+      case 7: M68000_AddCycles(34); break;    /* TRAPV */
+      case 8: M68000_AddCycles(34); break;    /* Privilege violation */
+      case 9: M68000_AddCycles(34); break;    /* Trace */
+      case 10: M68000_AddCycles(34); break;   /* Line-A - probably wrong */
+      case 11: M68000_AddCycles(34); break;   /* Line-F - probably wrong */
       default:
         /* FIXME: Add right cycles value for MFP interrupts and copro exceptions ... */
         if(nr < 64)
-          ADD_CYCLES(0, 0, 0);       /* Coprocessor and unassigned exceptions (???) */
+          M68000_AddCycles(4);       /* Coprocessor and unassigned exceptions (???) */
         else
-          ADD_CYCLES(24, 0, 0);      /* Must be a MFP interrupt */
+          M68000_AddCycles(24);      /* Must be a MFP interrupt */
         break;
     }
 }
@@ -1157,9 +1183,6 @@ void m68k_mull (uae_u32 opcode, uae_u32 src, uae_u16 extra)
 #endif
 }
 
-static char* ccnames[] =
-{ "T ","F ","HI","LS","CC","CS","NE","EQ",
-  "VC","VS","PL","MI","GE","LT","GT","LE" };
 
 void m68k_reset (void)
 {
@@ -1186,8 +1209,9 @@ void m68k_reset (void)
 
 unsigned long REGPARAM2 op_illg (uae_u32 opcode)
 {
+#if 0
     uaecptr pc = m68k_getpc ();
-
+#endif
     if ((opcode & 0xF000) == 0xF000) {
 	Exception(0xB,0);
 	return 4;
@@ -1196,13 +1220,6 @@ unsigned long REGPARAM2 op_illg (uae_u32 opcode)
 	Exception(0xA,0);
 	return 4;
     }
-
-    if (get_long (0x10) == 0) {
-	write_log("Illegal instruction handler failure!\n");
-	set_special(SPCFLAG_BRK);
-	bQuitProgram = 1;
-    }
-
 #if 0
     write_log ("Illegal instruction: %04x at %08lx\n", opcode, (long)pc);
 #endif
@@ -1210,6 +1227,7 @@ unsigned long REGPARAM2 op_illg (uae_u32 opcode)
     return 4;
 }
 
+
 void mmu_op(uae_u32 opcode, uae_u16 extra)
 {
     if ((opcode & 0xFE0) == 0x0500) {
@@ -1262,26 +1280,42 @@ static void do_trace (void)
 }
 
 
+/*
+ * Handle special flags
+ */
 static int do_specialties (void)
 {
     if(regs.spcflags & SPCFLAG_BUSERROR) {
 	/* We can not execute bus errors directly in the memory handler
 	 * functions since the PC should point to the address of the next
 	 * instruction, so we're executing the bus errors here: */
-        unset_special(SPCFLAG_BUSERROR);
+	unset_special(SPCFLAG_BUSERROR);
 	Exception(2,0);
     }
 
+    if(regs.spcflags & SPCFLAG_EXTRA_CYCLES) {
+	/* Add some extra cycles to simulate a wait state */
+	unset_special(SPCFLAG_EXTRA_CYCLES);
+	M68000_AddCycles(4);
+    }
+
     if (regs.spcflags & SPCFLAG_DOTRACE) {
 	Exception (9,last_trace_ad);
     }
+
     while (regs.spcflags & SPCFLAG_STOP) {
-	do_cycles (8);
-	if (regs.intmask>5) {
-            /* We still have to care about events when IPL==7 ! */
-            Main_EventHandler();
-            if(bQuitProgram)  unset_special(SPCFLAG_STOP);
-        }
+	if (regs.intmask > 5) {
+	    /* We still have to care about events when IPL==7 ! */
+	    Main_EventHandler();
+	    if (regs.spcflags & SPCFLAG_BRK)  return 1;
+	}
+	M68000_AddCycles(4);
+	if (PendingInterruptCount<=0 && PendingInterruptFunction) {
+	    CALL_VAR(PendingInterruptFunction);
+	}
+	if (regs.spcflags & SPCFLAG_MFP) {
+	    MFP_CheckPendingInterrupts();
+	}
 	if (regs.spcflags & (SPCFLAG_INT | SPCFLAG_DOINT)) {
 	    int intr = intlev ();
 	    unset_special (SPCFLAG_INT | SPCFLAG_DOINT);
@@ -1292,6 +1326,7 @@ static int do_specialties (void)
 	    }
 	}
     }
+
     if (regs.spcflags & SPCFLAG_TRACE)
 	do_trace ();
 
@@ -1308,10 +1343,16 @@ static int do_specialties (void)
 	unset_special (SPCFLAG_INT);
 	set_special (SPCFLAG_DOINT);
     }
+
+    if (regs.spcflags & SPCFLAG_MFP) {          /* Check for MFP interrupts */
+	MFP_CheckPendingInterrupts();
+    }
+
     if (regs.spcflags & (SPCFLAG_BRK | SPCFLAG_MODE_CHANGE)) {
-	unset_special (SPCFLAG_BRK | SPCFLAG_MODE_CHANGE);
+	unset_special(SPCFLAG_MODE_CHANGE);
 	return 1;
     }
+
     return 0;
 }
 
@@ -1324,9 +1365,11 @@ static void m68k_run_1 (void)
     uae_u8 saved_bytes[20];
     uae_u16 *oldpcp;
 #endif
-    while(!bQuitProgram) {
+
+    for (;;) {
 	int cycles;
 	uae_u32 opcode = get_iword_prefetch (0);
+
 #ifdef DEBUG_PREFETCH
 	if (get_ilong (0) != do_get_mem_long (&regs.prefetch)) {
 	    fprintf (stderr, "Prefetch differs from memory.\n");
@@ -1359,7 +1402,10 @@ static void m68k_run_1 (void)
 	}
 #endif
 
-	do_cycles (cycles);
+	M68000_AddCycles(cycles);
+	if (PendingInterruptCount<=0 && PendingInterruptFunction)
+	  CALL_VAR(PendingInterruptFunction);
+
 	if (regs.spcflags) {
 	    if (do_specialties ())
 		return;
@@ -1371,7 +1417,7 @@ static void m68k_run_1 (void)
 /* Same thing, but don't use prefetch to get opcode.  */
 static void m68k_run_2 (void)
 {
-    while(!bQuitProgram) {
+    for (;;) {
 	int cycles;
 	uae_u32 opcode = get_iword (0);
 
@@ -1389,7 +1435,10 @@ static void m68k_run_2 (void)
 
 	cycles = (*cpufunctbl[opcode])(opcode);
 
-	do_cycles (cycles);
+	M68000_AddCycles(cycles);
+	if (PendingInterruptCount<=0 && PendingInterruptFunction)
+	  CALL_VAR(PendingInterruptFunction);
+
 	if (regs.spcflags) {
 	    if (do_specialties ())
 		return;
@@ -1398,25 +1447,28 @@ static void m68k_run_2 (void)
 }
 
 
-int in_m68k_go = 0;
-
 void m68k_go (int may_quit)
 {
+    static int in_m68k_go = 0;
+
     if (in_m68k_go || !may_quit) {
 	write_log ("Bug! m68k_go is not reentrant.\n");
 	abort ();
     }
 
     in_m68k_go++;
-    while(!bQuitProgram) {
+    while (!(regs.spcflags & SPCFLAG_BRK)) {
         if(cpu_compatible)
           m68k_run_1();
          else
           m68k_run_2();
     }
+    unset_special(SPCFLAG_BRK);
     in_m68k_go--;
 }
 
+
+/*
 static void m68k_verify (uaecptr addr, uaecptr *nextpc)
 {
     uae_u32 opcode, val;
@@ -1444,9 +1496,15 @@ static void m68k_verify (uaecptr addr, uaecptr *nextpc)
 	}
     }
 }
+*/
+
 
 void m68k_disasm (FILE *f, uaecptr addr, uaecptr *nextpc, int cnt)
 {
+    static const char* ccnames[] =
+        { "T ","F ","HI","LS","CC","CS","NE","EQ",
+          "VC","VS","PL","MI","GE","LT","GT","LE" };
+
     uaecptr newpc = 0;
     m68kpc_offset = addr - m68k_getpc ();
     while (cnt-- > 0) {
