@@ -1,18 +1,19 @@
 /*
-  Hatari
+  Hatari - vdi.c
+
+  This file is distributed under the GNU Public License, version 2 or at
+  your option any later version. Read the file gpl.txt for details.
 
   VDI (Virtual Device Interface) (Trap #2)
 
-  To get higher resolutions on the Desktop, we intercept the VDI/Line-A calls and set elements
-  in their structures to the higher width/height/cel/planes. We need to intercept the initial Line-A
-  call(which we force into the TOS on boot-up) and also the init calls to the VDI.
-  As we intercept the VDI calls, this is a good point to pass them on to the accelerated native
-  PC functions if we need to - this improves drawing speed far greater than any ST software
-  accelerator.
+  To get higher resolutions on the Desktop, we intercept the VDI/Line-A calls
+  and set elements in their structures to the higher width/height/cel/planes.
+  We need to intercept the initial Line-A call (which we force into the TOS on
+  boot-up) and also the init calls to the VDI.
 */
+char VDI_rcsid[] = "Hatari $Id: vdi.c,v 1.11 2004/04/23 15:33:59 thothy Exp $";
 
 #include "main.h"
-#include "decode.h"
 #include "file.h"
 #include "gemdos.h"
 #include "m68000.h"
@@ -235,7 +236,7 @@ void VDI_Complete(void)
 /*
   Save desktop configuration file for VDI, eg desktop.inf(TOS 1.04) or newdesk.inf(TOS 2.06)
 */
-void VDI_SaveDesktopInf(char *pszFileName,unsigned char *Script,long ScriptSize)
+static void VDI_SaveDesktopInf(char *pszFileName,unsigned char *Script,long ScriptSize)
 {
   /* Just save file */
   File_Save(pszFileName, Script, ScriptSize, FALSE);
@@ -246,7 +247,7 @@ void VDI_SaveDesktopInf(char *pszFileName,unsigned char *Script,long ScriptSize)
 /*
   Modify exisiting ST desktop configuration files to set resolution(keep user settings)
 */
-void VDI_ModifyDesktopInf(char *pszFileName)
+static void VDI_ModifyDesktopInf(char *pszFileName)
 {
   long InfSize;
   unsigned char *pInfData;
@@ -286,7 +287,7 @@ done_modify:;
 */
 void VDI_FixDesktopInf(void)
 {
-  char szDesktopFileName[MAX_FILENAME_LENGTH],szNewDeskFileName[MAX_FILENAME_LENGTH];
+  char *szDesktopFileName, *szNewDeskFileName;
 
   if(!GEMDOS_EMU_ON)
   {
@@ -294,6 +295,9 @@ void VDI_FixDesktopInf(void)
     return;
   }
 
+  szDesktopFileName = Memory_Alloc(2 * FILENAME_MAX);
+  szNewDeskFileName = szDesktopFileName + FILENAME_MAX;
+
   /* Create filenames for hard-drive */
   GemDOS_CreateHardDriveFileName(2, "\\DESKTOP.INF", szDesktopFileName);
   GemDOS_CreateHardDriveFileName(2, "\\NEWDESK.INF", szNewDeskFileName);
@@ -306,4 +310,6 @@ void VDI_FixDesktopInf(void)
   if (!File_Exists(szNewDeskFileName))
     VDI_SaveDesktopInf(szNewDeskFileName,NewDeskScript,sizeof(NewDeskScript));
   VDI_ModifyDesktopInf(szNewDeskFileName);
+
+  Memory_Free(szDesktopFileName);
 }
