@@ -9,13 +9,13 @@
   TV raster trace, border removal, palette changes per HBL, the 'video address
   pointer' etc...
 */
-char Video_rcsid[] = "Hatari $Id: video.c,v 1.20 2003/10/18 07:46:55 thothy Exp $";
+char Video_rcsid[] = "Hatari $Id: video.c,v 1.24 2004/06/11 10:04:46 thothy Exp $";
 
 #include <SDL.h>
+#include <SDL_endian.h>
 
 #include "main.h"
 #include "debug.h"
-#include "decode.h"
 #include "configuration.h"
 #include "fdc.h"
 #include "int.h"
@@ -236,8 +236,11 @@ void Video_InterruptHandler_VBL(void)
 
   /* And handle any messages, check for quit message */
   Main_EventHandler();         /* Process messages, set 'bQuitProgram' if user tries to quit */
-  if(bQuitProgram)
+  if (bQuitProgram)
+  {
     Int_AddAbsoluteInterrupt(4, 0L);  /* Pass NULL interrupt function to quit cleanly */
+    set_special(SPCFLAG_BRK);         /* Assure that CPU core shuts down */
+  }
 
   if (ConfigureParams.System.nMinMaxSpeed != MINMAXSPEED_MAX)
   {
@@ -417,14 +420,14 @@ void Video_StartHBL(void)
 /*
   Store whole palette on first line so have reference to work from
 */
-void Video_StoreFirstLinePalette(void)
+static void Video_StoreFirstLinePalette(void)
 {
   unsigned short int *pp2;
   int i;
 
   pp2 = (unsigned short int *)((unsigned long)STRam+0xff8240);
   for(i=0; i<16; i++)
-    HBLPalettes[i] = STMemory_Swap68000Int(*pp2++);
+    HBLPalettes[i] = SDL_SwapBE16(*pp2++);
   /* And set mask flag with palette and resolution */
   HBLPaletteMasks[0] = (PALETTEMASK_RESOLUTION|PALETTEMASK_PALETTE) | (((unsigned long)STMemory_ReadByte(0xff8260)&0x3)<<16);
 }
@@ -434,7 +437,7 @@ void Video_StoreFirstLinePalette(void)
 /*
   Store resolution on each line(used to test if mixed low/medium resolutions)
 */
-void Video_StoreResolution(int y)
+static void Video_StoreResolution(int y)
 {
   /* Clear resolution, and set with current value */
   if (!(bUseHighRes || bUseVDIRes) ) {
@@ -449,7 +452,7 @@ void Video_StoreResolution(int y)
   Copy line of screen into buffer for conversion later.
   Possible lines may be top/bottom border, and/or left/right borders
 */
-void Video_CopyScreenLine(int BorderMask)
+static void Video_CopyScreenLine(int BorderMask)
 {
   /* Only copy screen line if not doing high VDI resolution */
   if (!bUseVDIRes) {
